# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_ATROPOS_N4_SH_METADATA = Metadata(
    id="3902130432999f1a99133857c02c3377a225b17e.boutiques",
    name="antsAtroposN4.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsAtroposN4ShSegmentationPriorsParamsDictNoTag = typing.TypedDict('_AntsAtroposN4ShSegmentationPriorsParamsDictNoTag', {
    "segmentation_priors_pattern": typing.NotRequired[str | None],
    "segmentation_priors_folder": typing.NotRequired[InputPathType | None],
})
AntsAtroposN4ShSegmentationPriorsParamsDictTagged = typing.TypedDict('AntsAtroposN4ShSegmentationPriorsParamsDictTagged', {
    "@type": typing.Literal["segmentation_priors"],
    "segmentation_priors_pattern": typing.NotRequired[str | None],
    "segmentation_priors_folder": typing.NotRequired[InputPathType | None],
})
AntsAtroposN4ShSegmentationPriorsParamsDict = _AntsAtroposN4ShSegmentationPriorsParamsDictNoTag | AntsAtroposN4ShSegmentationPriorsParamsDictTagged


_AntsAtroposN4ShParamsDictNoTag = typing.TypedDict('_AntsAtroposN4ShParamsDictNoTag', {
    "image_dimension": typing.Literal[2, 3],
    "input_image": InputPathType,
    "mask_image": InputPathType,
    "number_of_classes": int,
    "output_prefix": str,
    "max_n4_atropos_iterations": typing.NotRequired[int | None],
    "max_atropos_iterations": typing.NotRequired[int | None],
    "segmentation_priors": AntsAtroposN4ShSegmentationPriorsParamsDict,
    "mrf": typing.NotRequired[str | None],
    "denoise_anatomical_images": typing.NotRequired[bool | None],
    "posterior_formulation": typing.NotRequired[typing.Literal["Socrates[ 1 ]", "Aristotle[ 1 ]"] | None],
    "label_propagation": typing.NotRequired[str | None],
    "posterior_label_for_n4_weight_mask": typing.NotRequired[str | None],
    "image_file_suffix": typing.NotRequired[str | None],
    "keep_temporary_files": typing.NotRequired[bool | None],
    "use_random_seeding": typing.NotRequired[bool | None],
    "atropos_segmentation_prior_weight": typing.NotRequired[float | None],
    "n4_convergence": typing.NotRequired[str | None],
    "n4_shrink_factor": typing.NotRequired[int | None],
    "n4_bspline_params": typing.NotRequired[str | None],
    "atropos_segmentation_icm": typing.NotRequired[str | None],
    "atropos_segmentation_use_euclidean_distance": typing.NotRequired[bool | None],
    "test_debug_mode": typing.NotRequired[int | None],
})
AntsAtroposN4ShParamsDictTagged = typing.TypedDict('AntsAtroposN4ShParamsDictTagged', {
    "@type": typing.Literal["ants/antsAtroposN4.sh"],
    "image_dimension": typing.Literal[2, 3],
    "input_image": InputPathType,
    "mask_image": InputPathType,
    "number_of_classes": int,
    "output_prefix": str,
    "max_n4_atropos_iterations": typing.NotRequired[int | None],
    "max_atropos_iterations": typing.NotRequired[int | None],
    "segmentation_priors": AntsAtroposN4ShSegmentationPriorsParamsDict,
    "mrf": typing.NotRequired[str | None],
    "denoise_anatomical_images": typing.NotRequired[bool | None],
    "posterior_formulation": typing.NotRequired[typing.Literal["Socrates[ 1 ]", "Aristotle[ 1 ]"] | None],
    "label_propagation": typing.NotRequired[str | None],
    "posterior_label_for_n4_weight_mask": typing.NotRequired[str | None],
    "image_file_suffix": typing.NotRequired[str | None],
    "keep_temporary_files": typing.NotRequired[bool | None],
    "use_random_seeding": typing.NotRequired[bool | None],
    "atropos_segmentation_prior_weight": typing.NotRequired[float | None],
    "n4_convergence": typing.NotRequired[str | None],
    "n4_shrink_factor": typing.NotRequired[int | None],
    "n4_bspline_params": typing.NotRequired[str | None],
    "atropos_segmentation_icm": typing.NotRequired[str | None],
    "atropos_segmentation_use_euclidean_distance": typing.NotRequired[bool | None],
    "test_debug_mode": typing.NotRequired[int | None],
})
AntsAtroposN4ShParamsDict = _AntsAtroposN4ShParamsDictNoTag | AntsAtroposN4ShParamsDictTagged


def ants_atropos_n4_sh_segmentation_priors(
    segmentation_priors_pattern: str | None = None,
    segmentation_priors_folder: InputPathType | None = None,
) -> AntsAtroposN4ShSegmentationPriorsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        segmentation_priors_pattern: Prior probability images initializing the\
            segmentation. Specified using c-style formatting, e.g. -p\
            labelsPriors%02d.nii.gz. If this is not specified, k-means\
            initialization is used instead.
        segmentation_priors_folder: Included so.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "segmentation_priors",
    }
    if segmentation_priors_pattern is not None:
        params["segmentation_priors_pattern"] = segmentation_priors_pattern
    if segmentation_priors_folder is not None:
        params["segmentation_priors_folder"] = segmentation_priors_folder
    return params


def ants_atropos_n4_sh_segmentation_priors_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsAtroposN4ShSegmentationPriorsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("segmentation_priors_pattern", None) is not None:
        if not isinstance(params["segmentation_priors_pattern"], str):
            raise StyxValidationError(f'`segmentation_priors_pattern` has the wrong type: Received `{type(params.get("segmentation_priors_pattern", None))}` expected `str | None`')
    if params.get("segmentation_priors_folder", None) is not None:
        if not isinstance(params["segmentation_priors_folder"], (pathlib.Path, str)):
            raise StyxValidationError(f'`segmentation_priors_folder` has the wrong type: Received `{type(params.get("segmentation_priors_folder", None))}` expected `InputPathType | None`')


def ants_atropos_n4_sh_segmentation_priors_cargs(
    params: AntsAtroposN4ShSegmentationPriorsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("segmentation_priors_pattern", None) is not None or params.get("segmentation_priors_folder", None) is not None:
        cargs.append((params.get("segmentation_priors_pattern", None) if (params.get("segmentation_priors_pattern", None) is not None) else "") + "/" + (execution.input_file(params.get("segmentation_priors_folder", None)) if (params.get("segmentation_priors_folder", None) is not None) else ""))
    return cargs


class AntsAtroposN4ShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsAtroposN4ShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    n4_corrected: OutputPathType
    """N4 corrected image."""
    segmentation: OutputPathType
    """Segmentation image."""
    segmentation_posteriors: OutputPathType
    """Segmentation posteriors image."""


def ants_atropos_n4_sh_params(
    image_dimension: typing.Literal[2, 3],
    input_image: InputPathType,
    mask_image: InputPathType,
    number_of_classes: int,
    output_prefix: str,
    segmentation_priors: AntsAtroposN4ShSegmentationPriorsParamsDict,
    max_n4_atropos_iterations: int | None = None,
    max_atropos_iterations: int | None = None,
    mrf: str | None = None,
    denoise_anatomical_images: bool | None = None,
    posterior_formulation: typing.Literal["Socrates[ 1 ]", "Aristotle[ 1 ]"] | None = None,
    label_propagation: str | None = None,
    posterior_label_for_n4_weight_mask: str | None = None,
    image_file_suffix: str | None = None,
    keep_temporary_files: bool | None = None,
    use_random_seeding: bool | None = None,
    atropos_segmentation_prior_weight: float | None = None,
    n4_convergence: str | None = None,
    n4_shrink_factor: int | None = None,
    n4_bspline_params: str | None = None,
    atropos_segmentation_icm: str | None = None,
    atropos_segmentation_use_euclidean_distance: bool | None = None,
    test_debug_mode: int | None = None,
) -> AntsAtroposN4ShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: 2 or 3, for 2- or 3-dimensional image.
        input_image: Anatomical image, typically T1. If more than one\
            anatomical image is specified, subsequent images are also used during\
            the segmentation process.
        mask_image: Binary mask defining the region of interest.
        number_of_classes: Number of classes defining the segmentation.
        output_prefix: The following images are created:\
            {output_prefix}N4Corrected.{output_suffix},\
            {output_prefix}Segmentation.{output_suffix},\
            {output_prefix}SegmentationPosteriors.{output_suffix}.
        segmentation_priors: Prior probability images initializing the\
            segmentation. Specified using c-style formatting, e.g. -p\
            labelsPriors%02d.nii.gz. If this is not specified, k-means\
            initialization is used instead.
        max_n4_atropos_iterations: Maximum number of (outer loop) iterations\
            between N4 <-> Atropos (default = 15).
        max_atropos_iterations: Maximum number of (inner loop) iterations in\
            Atropos (default = 3).
        mrf: Specifies MRF prior (of the form '[ weight,neighborhood ]', e.g.\
            '[ 0.1,1x1x1 ]' which is default).
        denoise_anatomical_images: Denoise anatomical images (1) or not (0)\
            (default = 1).
        posterior_formulation: Posterior formulation and whether or not to use\
            mixture model proportions. e.g 'Socrates[ 1 ]' (default) or 'Aristotle[\
            1 ]'. Choose the latter if you want to use the distance priors, see\
            also the -l option for label propagation control (default = 'Socrates[\
            1 ]').
        label_propagation: Incorporate a distance prior into the 'Aristotle'\
            posterior formulation. Should be of the form 'label[\
            lambda,boundaryProbability ]' where label is a value of 1,2,3,...\
            denoting label ID. The label probability for anything outside the\
            current label\
            \
            = boundaryProbability * exp( -lambda * distanceFromBoundary )\
            \
            Intuitively, smaller lambda values will increase the spatial\
            capture range of the distance prior. To apply to all label values,\
            simply omit specifying the label, i.e. -l '[\
            lambda,boundaryProbability ]'.
        posterior_label_for_n4_weight_mask: Which posterior probability image\
            should be used to define the N4 weight mask. Can also specify multiple\
            posteriors in which case the chosen posteriors are combined.
        image_file_suffix: Any of the standard ITK IO formats e.g. nrrd, nii.gz\
            (default), mhd.
        keep_temporary_files: Keep temporary files on disk (1) or delete them\
            (0) (default = 0).
        use_random_seeding: Use random number generated from system clock in\
            Atropos (default = 1).
        atropos_segmentation_prior_weight: Atropos spatial prior probability\
            weight for the segmentation (default = 0.25).
        n4_convergence: Convergence parameters for N4, see '-c' option in\
            N4BiasFieldCorrection (default = [50x50x50x50,0.0000001]).
        n4_shrink_factor: Shrink factor for N4 (default = 4).
        n4_bspline_params: N4 b-spline specification, see '-b' option in\
            N4BiasFieldCorrection (default = [200,0,0,0]).
        atropos_segmentation_icm: ICM parameters for segmentation, see '-g'\
            option in Atropos (default = [1,1]).
        atropos_segmentation_use_euclidean_distance: Use euclidean distances in\
            distance prior formulation (1) or not (0), see Atropos usage for\
            details (default = 1).
        test_debug_mode: If > 0, attempts to continue after errors.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsAtroposN4.sh",
        "image_dimension": image_dimension,
        "input_image": input_image,
        "mask_image": mask_image,
        "number_of_classes": number_of_classes,
        "output_prefix": output_prefix,
        "segmentation_priors": segmentation_priors,
    }
    if max_n4_atropos_iterations is not None:
        params["max_n4_atropos_iterations"] = max_n4_atropos_iterations
    if max_atropos_iterations is not None:
        params["max_atropos_iterations"] = max_atropos_iterations
    if mrf is not None:
        params["mrf"] = mrf
    if denoise_anatomical_images is not None:
        params["denoise_anatomical_images"] = denoise_anatomical_images
    if posterior_formulation is not None:
        params["posterior_formulation"] = posterior_formulation
    if label_propagation is not None:
        params["label_propagation"] = label_propagation
    if posterior_label_for_n4_weight_mask is not None:
        params["posterior_label_for_n4_weight_mask"] = posterior_label_for_n4_weight_mask
    if image_file_suffix is not None:
        params["image_file_suffix"] = image_file_suffix
    if keep_temporary_files is not None:
        params["keep_temporary_files"] = keep_temporary_files
    if use_random_seeding is not None:
        params["use_random_seeding"] = use_random_seeding
    if atropos_segmentation_prior_weight is not None:
        params["atropos_segmentation_prior_weight"] = atropos_segmentation_prior_weight
    if n4_convergence is not None:
        params["n4_convergence"] = n4_convergence
    if n4_shrink_factor is not None:
        params["n4_shrink_factor"] = n4_shrink_factor
    if n4_bspline_params is not None:
        params["n4_bspline_params"] = n4_bspline_params
    if atropos_segmentation_icm is not None:
        params["atropos_segmentation_icm"] = atropos_segmentation_icm
    if atropos_segmentation_use_euclidean_distance is not None:
        params["atropos_segmentation_use_euclidean_distance"] = atropos_segmentation_use_euclidean_distance
    if test_debug_mode is not None:
        params["test_debug_mode"] = test_debug_mode
    return params


def ants_atropos_n4_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsAtroposN4ShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `typing.Literal[2, 3]`')
    if params["image_dimension"] not in [2, 3]:
        raise StyxValidationError("Parameter `image_dimension` must be one of [2, 3]")
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("mask_image", None) is None:
        raise StyxValidationError("`mask_image` must not be None")
    if not isinstance(params["mask_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType`')
    if params.get("number_of_classes", None) is None:
        raise StyxValidationError("`number_of_classes` must not be None")
    if not isinstance(params["number_of_classes"], int):
        raise StyxValidationError(f'`number_of_classes` has the wrong type: Received `{type(params.get("number_of_classes", None))}` expected `int`')
    if params.get("output_prefix", None) is None:
        raise StyxValidationError("`output_prefix` must not be None")
    if not isinstance(params["output_prefix"], str):
        raise StyxValidationError(f'`output_prefix` has the wrong type: Received `{type(params.get("output_prefix", None))}` expected `str`')
    if params.get("max_n4_atropos_iterations", None) is not None:
        if not isinstance(params["max_n4_atropos_iterations"], int):
            raise StyxValidationError(f'`max_n4_atropos_iterations` has the wrong type: Received `{type(params.get("max_n4_atropos_iterations", None))}` expected `int | None`')
        if params["max_n4_atropos_iterations"] < 1:
            raise StyxValidationError("Parameter `max_n4_atropos_iterations` must be at least 1")
    if params.get("max_atropos_iterations", None) is not None:
        if not isinstance(params["max_atropos_iterations"], int):
            raise StyxValidationError(f'`max_atropos_iterations` has the wrong type: Received `{type(params.get("max_atropos_iterations", None))}` expected `int | None`')
        if params["max_atropos_iterations"] < 1:
            raise StyxValidationError("Parameter `max_atropos_iterations` must be at least 1")
    if params.get("segmentation_priors", None) is None:
        raise StyxValidationError("`segmentation_priors` must not be None")
    ants_atropos_n4_sh_segmentation_priors_validate(params["segmentation_priors"])
    if params.get("mrf", None) is not None:
        if not isinstance(params["mrf"], str):
            raise StyxValidationError(f'`mrf` has the wrong type: Received `{type(params.get("mrf", None))}` expected `str | None`')
    if params.get("denoise_anatomical_images", None) is not None:
        if not isinstance(params["denoise_anatomical_images"], bool):
            raise StyxValidationError(f'`denoise_anatomical_images` has the wrong type: Received `{type(params.get("denoise_anatomical_images", None))}` expected `bool | None`')
    if params.get("posterior_formulation", None) is not None:
        if not isinstance(params["posterior_formulation"], str):
            raise StyxValidationError(f'`posterior_formulation` has the wrong type: Received `{type(params.get("posterior_formulation", None))}` expected `typing.Literal["Socrates[ 1 ]", "Aristotle[ 1 ]"] | None`')
        if params["posterior_formulation"] not in ["Socrates[ 1 ]", "Aristotle[ 1 ]"]:
            raise StyxValidationError("Parameter `posterior_formulation` must be one of [\"Socrates[ 1 ]\", \"Aristotle[ 1 ]\"]")
    if params.get("label_propagation", None) is not None:
        if not isinstance(params["label_propagation"], str):
            raise StyxValidationError(f'`label_propagation` has the wrong type: Received `{type(params.get("label_propagation", None))}` expected `str | None`')
    if params.get("posterior_label_for_n4_weight_mask", None) is not None:
        if not isinstance(params["posterior_label_for_n4_weight_mask"], str):
            raise StyxValidationError(f'`posterior_label_for_n4_weight_mask` has the wrong type: Received `{type(params.get("posterior_label_for_n4_weight_mask", None))}` expected `str | None`')
    if params.get("image_file_suffix", None) is not None:
        if not isinstance(params["image_file_suffix"], str):
            raise StyxValidationError(f'`image_file_suffix` has the wrong type: Received `{type(params.get("image_file_suffix", None))}` expected `str | None`')
    if params.get("keep_temporary_files", None) is not None:
        if not isinstance(params["keep_temporary_files"], bool):
            raise StyxValidationError(f'`keep_temporary_files` has the wrong type: Received `{type(params.get("keep_temporary_files", None))}` expected `bool | None`')
    if params.get("use_random_seeding", None) is not None:
        if not isinstance(params["use_random_seeding"], bool):
            raise StyxValidationError(f'`use_random_seeding` has the wrong type: Received `{type(params.get("use_random_seeding", None))}` expected `bool | None`')
    if params.get("atropos_segmentation_prior_weight", None) is not None:
        if not isinstance(params["atropos_segmentation_prior_weight"], (float, int)):
            raise StyxValidationError(f'`atropos_segmentation_prior_weight` has the wrong type: Received `{type(params.get("atropos_segmentation_prior_weight", None))}` expected `float | None`')
    if params.get("n4_convergence", None) is not None:
        if not isinstance(params["n4_convergence"], str):
            raise StyxValidationError(f'`n4_convergence` has the wrong type: Received `{type(params.get("n4_convergence", None))}` expected `str | None`')
    if params.get("n4_shrink_factor", None) is not None:
        if not isinstance(params["n4_shrink_factor"], int):
            raise StyxValidationError(f'`n4_shrink_factor` has the wrong type: Received `{type(params.get("n4_shrink_factor", None))}` expected `int | None`')
    if params.get("n4_bspline_params", None) is not None:
        if not isinstance(params["n4_bspline_params"], str):
            raise StyxValidationError(f'`n4_bspline_params` has the wrong type: Received `{type(params.get("n4_bspline_params", None))}` expected `str | None`')
    if params.get("atropos_segmentation_icm", None) is not None:
        if not isinstance(params["atropos_segmentation_icm"], str):
            raise StyxValidationError(f'`atropos_segmentation_icm` has the wrong type: Received `{type(params.get("atropos_segmentation_icm", None))}` expected `str | None`')
    if params.get("atropos_segmentation_use_euclidean_distance", None) is not None:
        if not isinstance(params["atropos_segmentation_use_euclidean_distance"], bool):
            raise StyxValidationError(f'`atropos_segmentation_use_euclidean_distance` has the wrong type: Received `{type(params.get("atropos_segmentation_use_euclidean_distance", None))}` expected `bool | None`')
    if params.get("test_debug_mode", None) is not None:
        if not isinstance(params["test_debug_mode"], int):
            raise StyxValidationError(f'`test_debug_mode` has the wrong type: Received `{type(params.get("test_debug_mode", None))}` expected `int | None`')


def ants_atropos_n4_sh_cargs(
    params: AntsAtroposN4ShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsAtroposN4.sh")
    cargs.extend([
        "-d",
        str(params.get("image_dimension", None))
    ])
    cargs.extend([
        "-a",
        execution.input_file(params.get("input_image", None))
    ])
    cargs.extend([
        "-x",
        execution.input_file(params.get("mask_image", None))
    ])
    cargs.extend([
        "-c",
        str(params.get("number_of_classes", None))
    ])
    cargs.extend([
        "-o",
        params.get("output_prefix", None)
    ])
    if params.get("max_n4_atropos_iterations", None) is not None:
        cargs.extend([
            "-m",
            str(params.get("max_n4_atropos_iterations", None))
        ])
    if params.get("max_atropos_iterations", None) is not None:
        cargs.extend([
            "-n",
            str(params.get("max_atropos_iterations", None))
        ])
    cargs.extend([
        "-p",
        *ants_atropos_n4_sh_segmentation_priors_cargs(params.get("segmentation_priors", None), execution)
    ])
    if params.get("mrf", None) is not None:
        cargs.extend([
            "-r",
            params.get("mrf", None)
        ])
    if params.get("denoise_anatomical_images", None) is not None:
        cargs.extend([
            "-g",
            ("1" if params.get("denoise_anatomical_images", None) else "0")
        ])
    if params.get("posterior_formulation", None) is not None:
        cargs.extend([
            "-b",
            params.get("posterior_formulation", None)
        ])
    if params.get("label_propagation", None) is not None:
        cargs.extend([
            "-l",
            params.get("label_propagation", None)
        ])
    if params.get("posterior_label_for_n4_weight_mask", None) is not None:
        cargs.extend([
            "-y",
            params.get("posterior_label_for_n4_weight_mask", None)
        ])
    if params.get("image_file_suffix", None) is not None:
        cargs.extend([
            "-s",
            params.get("image_file_suffix", None)
        ])
    if params.get("keep_temporary_files", None) is not None:
        cargs.extend([
            "-k",
            ("1" if params.get("keep_temporary_files", None) else "0")
        ])
    if params.get("use_random_seeding", None) is not None:
        cargs.extend([
            "-u",
            ("1" if params.get("use_random_seeding", None) else "0")
        ])
    if params.get("atropos_segmentation_prior_weight", None) is not None:
        cargs.extend([
            "-w",
            str(params.get("atropos_segmentation_prior_weight", None))
        ])
    if params.get("n4_convergence", None) is not None:
        cargs.extend([
            "-e",
            params.get("n4_convergence", None)
        ])
    if params.get("n4_shrink_factor", None) is not None:
        cargs.extend([
            "-f",
            str(params.get("n4_shrink_factor", None))
        ])
    if params.get("n4_bspline_params", None) is not None:
        cargs.extend([
            "-q",
            params.get("n4_bspline_params", None)
        ])
    if params.get("atropos_segmentation_icm", None) is not None:
        cargs.extend([
            "-i",
            params.get("atropos_segmentation_icm", None)
        ])
    if params.get("atropos_segmentation_use_euclidean_distance", None) is not None:
        cargs.extend([
            "-j",
            ("1" if params.get("atropos_segmentation_use_euclidean_distance", None) else "0")
        ])
    if params.get("test_debug_mode", None) is not None:
        cargs.extend([
            "-z",
            str(params.get("test_debug_mode", None))
        ])
    return cargs


def ants_atropos_n4_sh_outputs(
    params: AntsAtroposN4ShParamsDict,
    execution: Execution,
) -> AntsAtroposN4ShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsAtroposN4ShOutputs(
        root=execution.output_file("."),
        n4_corrected=execution.output_file(params.get("output_prefix", None) + "N4Corrected.[OUTPUT_SUFFIX]"),
        segmentation=execution.output_file(params.get("output_prefix", None) + "Segmentation.[OUTPUT_SUFFIX]"),
        segmentation_posteriors=execution.output_file(params.get("output_prefix", None) + "SegmentationPosteriors.[OUTPUT_SUFFIX]"),
    )
    return ret


def ants_atropos_n4_sh_execute(
    params: AntsAtroposN4ShParamsDict,
    runner: Runner | None = None,
) -> AntsAtroposN4ShOutputs:
    """
    antsAtroposN4.sh
    
    antsAtroposN4.sh iterates between N4 <-> Atropos to improve segmentation
    results.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAtroposN4ShOutputs`).
    """
    ants_atropos_n4_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_ATROPOS_N4_SH_METADATA)
    params = execution.params(params)
    cargs = ants_atropos_n4_sh_cargs(params, execution)
    ret = ants_atropos_n4_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_atropos_n4_sh(
    image_dimension: typing.Literal[2, 3],
    input_image: InputPathType,
    mask_image: InputPathType,
    number_of_classes: int,
    output_prefix: str,
    segmentation_priors: AntsAtroposN4ShSegmentationPriorsParamsDict,
    max_n4_atropos_iterations: int | None = None,
    max_atropos_iterations: int | None = None,
    mrf: str | None = None,
    denoise_anatomical_images: bool | None = None,
    posterior_formulation: typing.Literal["Socrates[ 1 ]", "Aristotle[ 1 ]"] | None = None,
    label_propagation: str | None = None,
    posterior_label_for_n4_weight_mask: str | None = None,
    image_file_suffix: str | None = None,
    keep_temporary_files: bool | None = None,
    use_random_seeding: bool | None = None,
    atropos_segmentation_prior_weight: float | None = None,
    n4_convergence: str | None = None,
    n4_shrink_factor: int | None = None,
    n4_bspline_params: str | None = None,
    atropos_segmentation_icm: str | None = None,
    atropos_segmentation_use_euclidean_distance: bool | None = None,
    test_debug_mode: int | None = None,
    runner: Runner | None = None,
) -> AntsAtroposN4ShOutputs:
    """
    antsAtroposN4.sh
    
    antsAtroposN4.sh iterates between N4 <-> Atropos to improve segmentation
    results.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: 2 or 3, for 2- or 3-dimensional image.
        input_image: Anatomical image, typically T1. If more than one\
            anatomical image is specified, subsequent images are also used during\
            the segmentation process.
        mask_image: Binary mask defining the region of interest.
        number_of_classes: Number of classes defining the segmentation.
        output_prefix: The following images are created:\
            {output_prefix}N4Corrected.{output_suffix},\
            {output_prefix}Segmentation.{output_suffix},\
            {output_prefix}SegmentationPosteriors.{output_suffix}.
        segmentation_priors: Prior probability images initializing the\
            segmentation. Specified using c-style formatting, e.g. -p\
            labelsPriors%02d.nii.gz. If this is not specified, k-means\
            initialization is used instead.
        max_n4_atropos_iterations: Maximum number of (outer loop) iterations\
            between N4 <-> Atropos (default = 15).
        max_atropos_iterations: Maximum number of (inner loop) iterations in\
            Atropos (default = 3).
        mrf: Specifies MRF prior (of the form '[ weight,neighborhood ]', e.g.\
            '[ 0.1,1x1x1 ]' which is default).
        denoise_anatomical_images: Denoise anatomical images (1) or not (0)\
            (default = 1).
        posterior_formulation: Posterior formulation and whether or not to use\
            mixture model proportions. e.g 'Socrates[ 1 ]' (default) or 'Aristotle[\
            1 ]'. Choose the latter if you want to use the distance priors, see\
            also the -l option for label propagation control (default = 'Socrates[\
            1 ]').
        label_propagation: Incorporate a distance prior into the 'Aristotle'\
            posterior formulation. Should be of the form 'label[\
            lambda,boundaryProbability ]' where label is a value of 1,2,3,...\
            denoting label ID. The label probability for anything outside the\
            current label\
            \
            = boundaryProbability * exp( -lambda * distanceFromBoundary )\
            \
            Intuitively, smaller lambda values will increase the spatial\
            capture range of the distance prior. To apply to all label values,\
            simply omit specifying the label, i.e. -l '[\
            lambda,boundaryProbability ]'.
        posterior_label_for_n4_weight_mask: Which posterior probability image\
            should be used to define the N4 weight mask. Can also specify multiple\
            posteriors in which case the chosen posteriors are combined.
        image_file_suffix: Any of the standard ITK IO formats e.g. nrrd, nii.gz\
            (default), mhd.
        keep_temporary_files: Keep temporary files on disk (1) or delete them\
            (0) (default = 0).
        use_random_seeding: Use random number generated from system clock in\
            Atropos (default = 1).
        atropos_segmentation_prior_weight: Atropos spatial prior probability\
            weight for the segmentation (default = 0.25).
        n4_convergence: Convergence parameters for N4, see '-c' option in\
            N4BiasFieldCorrection (default = [50x50x50x50,0.0000001]).
        n4_shrink_factor: Shrink factor for N4 (default = 4).
        n4_bspline_params: N4 b-spline specification, see '-b' option in\
            N4BiasFieldCorrection (default = [200,0,0,0]).
        atropos_segmentation_icm: ICM parameters for segmentation, see '-g'\
            option in Atropos (default = [1,1]).
        atropos_segmentation_use_euclidean_distance: Use euclidean distances in\
            distance prior formulation (1) or not (0), see Atropos usage for\
            details (default = 1).
        test_debug_mode: If > 0, attempts to continue after errors.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAtroposN4ShOutputs`).
    """
    params = ants_atropos_n4_sh_params(
        image_dimension=image_dimension,
        input_image=input_image,
        mask_image=mask_image,
        number_of_classes=number_of_classes,
        output_prefix=output_prefix,
        max_n4_atropos_iterations=max_n4_atropos_iterations,
        max_atropos_iterations=max_atropos_iterations,
        segmentation_priors=segmentation_priors,
        mrf=mrf,
        denoise_anatomical_images=denoise_anatomical_images,
        posterior_formulation=posterior_formulation,
        label_propagation=label_propagation,
        posterior_label_for_n4_weight_mask=posterior_label_for_n4_weight_mask,
        image_file_suffix=image_file_suffix,
        keep_temporary_files=keep_temporary_files,
        use_random_seeding=use_random_seeding,
        atropos_segmentation_prior_weight=atropos_segmentation_prior_weight,
        n4_convergence=n4_convergence,
        n4_shrink_factor=n4_shrink_factor,
        n4_bspline_params=n4_bspline_params,
        atropos_segmentation_icm=atropos_segmentation_icm,
        atropos_segmentation_use_euclidean_distance=atropos_segmentation_use_euclidean_distance,
        test_debug_mode=test_debug_mode,
    )
    return ants_atropos_n4_sh_execute(params, runner)


__all__ = [
    "ANTS_ATROPOS_N4_SH_METADATA",
    "AntsAtroposN4ShOutputs",
    "AntsAtroposN4ShParamsDict",
    "AntsAtroposN4ShParamsDictTagged",
    "AntsAtroposN4ShSegmentationPriorsParamsDict",
    "AntsAtroposN4ShSegmentationPriorsParamsDictTagged",
    "ants_atropos_n4_sh",
    "ants_atropos_n4_sh_execute",
    "ants_atropos_n4_sh_params",
    "ants_atropos_n4_sh_segmentation_priors",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTSINTEGRATE_VECTOR_FIELD_METADATA = Metadata(
    id="834181e3e327a31ccc73d56b821e216f4579fc76.boutiques",
    name="ANTSIntegrateVectorField",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsintegrateVectorFieldParamsDictNoTag = typing.TypedDict('_AntsintegrateVectorFieldParamsDictNoTag', {
    "vector_field_input": InputPathType,
    "roi_mask_input": InputPathType,
    "fibers_output": str,
    "length_image_output": str,
})
AntsintegrateVectorFieldParamsDictTagged = typing.TypedDict('AntsintegrateVectorFieldParamsDictTagged', {
    "@type": typing.Literal["ants/ANTSIntegrateVectorField"],
    "vector_field_input": InputPathType,
    "roi_mask_input": InputPathType,
    "fibers_output": str,
    "length_image_output": str,
})
AntsintegrateVectorFieldParamsDict = _AntsintegrateVectorFieldParamsDictNoTag | AntsintegrateVectorFieldParamsDictTagged


class AntsintegrateVectorFieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsintegrateVectorFieldParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    fibers_out_vtk: OutputPathType
    """The output is the fibers as a VTK text file."""
    length_image_out_nii: OutputPathType
    """The output is the length image."""


def antsintegrate_vector_field_params(
    vector_field_input: InputPathType,
    roi_mask_input: InputPathType,
    fibers_output: str,
    length_image_output: str,
) -> AntsintegrateVectorFieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        vector_field_input: Input vector field image (e.g., VecImageIN.nii.gz),\
            where vectors are voxels.
        roi_mask_input: Input ROI mask image (e.g., ROIMaskIN.nii.gz), an\
            integer image controlling where the integration is performed.
        fibers_output: Output VTK text file for fibers (e.g., FibersOUT.vtk).
        length_image_output: Output length image (e.g., LengthImageOUT.nii.gz).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ANTSIntegrateVectorField",
        "vector_field_input": vector_field_input,
        "roi_mask_input": roi_mask_input,
        "fibers_output": fibers_output,
        "length_image_output": length_image_output,
    }
    return params


def antsintegrate_vector_field_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsintegrateVectorFieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("vector_field_input", None) is None:
        raise StyxValidationError("`vector_field_input` must not be None")
    if not isinstance(params["vector_field_input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`vector_field_input` has the wrong type: Received `{type(params.get("vector_field_input", None))}` expected `InputPathType`')
    if params.get("roi_mask_input", None) is None:
        raise StyxValidationError("`roi_mask_input` must not be None")
    if not isinstance(params["roi_mask_input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`roi_mask_input` has the wrong type: Received `{type(params.get("roi_mask_input", None))}` expected `InputPathType`')
    if params.get("fibers_output", None) is None:
        raise StyxValidationError("`fibers_output` must not be None")
    if not isinstance(params["fibers_output"], str):
        raise StyxValidationError(f'`fibers_output` has the wrong type: Received `{type(params.get("fibers_output", None))}` expected `str`')
    if params.get("length_image_output", None) is None:
        raise StyxValidationError("`length_image_output` must not be None")
    if not isinstance(params["length_image_output"], str):
        raise StyxValidationError(f'`length_image_output` has the wrong type: Received `{type(params.get("length_image_output", None))}` expected `str`')


def antsintegrate_vector_field_cargs(
    params: AntsintegrateVectorFieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ANTSIntegrateVectorField")
    cargs.append(execution.input_file(params.get("vector_field_input", None)))
    cargs.append(execution.input_file(params.get("roi_mask_input", None)))
    cargs.append(params.get("fibers_output", None))
    cargs.append(params.get("length_image_output", None))
    return cargs


def antsintegrate_vector_field_outputs(
    params: AntsintegrateVectorFieldParamsDict,
    execution: Execution,
) -> AntsintegrateVectorFieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsintegrateVectorFieldOutputs(
        root=execution.output_file("."),
        fibers_out_vtk=execution.output_file(params.get("fibers_output", None)),
        length_image_out_nii=execution.output_file(params.get("length_image_output", None)),
    )
    return ret


def antsintegrate_vector_field_execute(
    params: AntsintegrateVectorFieldParamsDict,
    runner: Runner | None = None,
) -> AntsintegrateVectorFieldOutputs:
    """
    ANTSIntegrateVectorField
    
    This tool integrates a vector field, where vectors are voxels, using a
    region of interest (ROI) mask. The ROI mask controls where the integration
    is performed and specifies the starting point region.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsintegrateVectorFieldOutputs`).
    """
    antsintegrate_vector_field_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSINTEGRATE_VECTOR_FIELD_METADATA)
    params = execution.params(params)
    cargs = antsintegrate_vector_field_cargs(params, execution)
    ret = antsintegrate_vector_field_outputs(params, execution)
    execution.run(cargs)
    return ret


def antsintegrate_vector_field(
    vector_field_input: InputPathType,
    roi_mask_input: InputPathType,
    fibers_output: str,
    length_image_output: str,
    runner: Runner | None = None,
) -> AntsintegrateVectorFieldOutputs:
    """
    ANTSIntegrateVectorField
    
    This tool integrates a vector field, where vectors are voxels, using a
    region of interest (ROI) mask. The ROI mask controls where the integration
    is performed and specifies the starting point region.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        vector_field_input: Input vector field image (e.g., VecImageIN.nii.gz),\
            where vectors are voxels.
        roi_mask_input: Input ROI mask image (e.g., ROIMaskIN.nii.gz), an\
            integer image controlling where the integration is performed.
        fibers_output: Output VTK text file for fibers (e.g., FibersOUT.vtk).
        length_image_output: Output length image (e.g., LengthImageOUT.nii.gz).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsintegrateVectorFieldOutputs`).
    """
    params = antsintegrate_vector_field_params(
        vector_field_input=vector_field_input,
        roi_mask_input=roi_mask_input,
        fibers_output=fibers_output,
        length_image_output=length_image_output,
    )
    return antsintegrate_vector_field_execute(params, runner)


__all__ = [
    "ANTSINTEGRATE_VECTOR_FIELD_METADATA",
    "AntsintegrateVectorFieldOutputs",
    "AntsintegrateVectorFieldParamsDict",
    "AntsintegrateVectorFieldParamsDictTagged",
    "antsintegrate_vector_field",
    "antsintegrate_vector_field_execute",
    "antsintegrate_vector_field_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CREATE_DISPLACEMENT_FIELD_METADATA = Metadata(
    id="be9ee3bddb712bf6b85e065f4aae1b6cbcda8c5e.boutiques",
    name="CreateDisplacementField",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_CreateDisplacementFieldParamsDictNoTag = typing.TypedDict('_CreateDisplacementFieldParamsDictNoTag', {
    "image_dimension": int,
    "enforce_zero_boundary_flag": bool,
    "component_images": list[InputPathType],
    "output_image": str,
})
CreateDisplacementFieldParamsDictTagged = typing.TypedDict('CreateDisplacementFieldParamsDictTagged', {
    "@type": typing.Literal["ants/CreateDisplacementField"],
    "image_dimension": int,
    "enforce_zero_boundary_flag": bool,
    "component_images": list[InputPathType],
    "output_image": str,
})
CreateDisplacementFieldParamsDict = _CreateDisplacementFieldParamsDictNoTag | CreateDisplacementFieldParamsDictTagged


class CreateDisplacementFieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CreateDisplacementFieldParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_displacement_field: OutputPathType
    """The generated itkImage of itkVector pixels representing the displacement
    field."""


def create_displacement_field_params(
    image_dimension: int,
    enforce_zero_boundary_flag: bool,
    component_images: list[InputPathType],
    output_image: str,
) -> CreateDisplacementFieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimension of the image, typically 2 or 3.
        enforce_zero_boundary_flag: Create zero-valued vectors along the\
            borders when enabled (pass 1), recommended for better displacement\
            field behavior.
        component_images: Input component images, each used for a vector\
            component. All component images must have the same size, offset,\
            origin, and spacing.
        output_image: The output displacement field image with itkVector\
            pixels.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/CreateDisplacementField",
        "image_dimension": image_dimension,
        "enforce_zero_boundary_flag": enforce_zero_boundary_flag,
        "component_images": component_images,
        "output_image": output_image,
    }
    return params


def create_displacement_field_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CreateDisplacementFieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("enforce_zero_boundary_flag", None) is None:
        raise StyxValidationError("`enforce_zero_boundary_flag` must not be None")
    if not isinstance(params["enforce_zero_boundary_flag"], bool):
        raise StyxValidationError(f'`enforce_zero_boundary_flag` has the wrong type: Received `{type(params.get("enforce_zero_boundary_flag", None))}` expected `bool`')
    if params.get("component_images", None) is None:
        raise StyxValidationError("`component_images` must not be None")
    if not isinstance(params["component_images"], list):
        raise StyxValidationError(f'`component_images` has the wrong type: Received `{type(params.get("component_images", None))}` expected `list[InputPathType]`')
    if not (1 <= len(params["component_images"]) <= 8):
        raise StyxValidationError("Parameter `component_images` must contain between 1 and 8 elements (inclusive)")
    for e in params["component_images"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`component_images` has the wrong type: Received `{type(params.get("component_images", None))}` expected `list[InputPathType]`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')


def create_displacement_field_cargs(
    params: CreateDisplacementFieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("CreateDisplacementField")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(("1" if params.get("enforce_zero_boundary_flag", None) else "0"))
    cargs.extend([execution.input_file(f) for f in params.get("component_images", None)])
    cargs.append(params.get("output_image", None))
    return cargs


def create_displacement_field_outputs(
    params: CreateDisplacementFieldParamsDict,
    execution: Execution,
) -> CreateDisplacementFieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CreateDisplacementFieldOutputs(
        root=execution.output_file("."),
        output_displacement_field=execution.output_file(params.get("output_image", None)),
    )
    return ret


def create_displacement_field_execute(
    params: CreateDisplacementFieldParamsDict,
    runner: Runner | None = None,
) -> CreateDisplacementFieldOutputs:
    """
    CreateDisplacementField
    
    Create an itkImage of itkVector pixels (NOT an itkVectorImage), using each
    scalar input component image for each vector component. An itkImage of
    itkVectors is the standard type for displacement fields in ITK. All
    component images (up to 8) are assumed to have the same size, offset,
    origin, and spacing. The 'EnforceZeroBoundaryFlag' option will create
    zero-valued vectors along the borders when enabled (pass 1), and is
    recommended for better displacement field behavior.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateDisplacementFieldOutputs`).
    """
    create_displacement_field_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CREATE_DISPLACEMENT_FIELD_METADATA)
    params = execution.params(params)
    cargs = create_displacement_field_cargs(params, execution)
    ret = create_displacement_field_outputs(params, execution)
    execution.run(cargs)
    return ret


def create_displacement_field(
    image_dimension: int,
    enforce_zero_boundary_flag: bool,
    component_images: list[InputPathType],
    output_image: str,
    runner: Runner | None = None,
) -> CreateDisplacementFieldOutputs:
    """
    CreateDisplacementField
    
    Create an itkImage of itkVector pixels (NOT an itkVectorImage), using each
    scalar input component image for each vector component. An itkImage of
    itkVectors is the standard type for displacement fields in ITK. All
    component images (up to 8) are assumed to have the same size, offset,
    origin, and spacing. The 'EnforceZeroBoundaryFlag' option will create
    zero-valued vectors along the borders when enabled (pass 1), and is
    recommended for better displacement field behavior.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimension of the image, typically 2 or 3.
        enforce_zero_boundary_flag: Create zero-valued vectors along the\
            borders when enabled (pass 1), recommended for better displacement\
            field behavior.
        component_images: Input component images, each used for a vector\
            component. All component images must have the same size, offset,\
            origin, and spacing.
        output_image: The output displacement field image with itkVector\
            pixels.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateDisplacementFieldOutputs`).
    """
    params = create_displacement_field_params(
        image_dimension=image_dimension,
        enforce_zero_boundary_flag=enforce_zero_boundary_flag,
        component_images=component_images,
        output_image=output_image,
    )
    return create_displacement_field_execute(params, runner)


__all__ = [
    "CREATE_DISPLACEMENT_FIELD_METADATA",
    "CreateDisplacementFieldOutputs",
    "CreateDisplacementFieldParamsDict",
    "CreateDisplacementFieldParamsDictTagged",
    "create_displacement_field",
    "create_displacement_field_execute",
    "create_displacement_field_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_MOTION_CORR_STATS_METADATA = Metadata(
    id="51fc61f7cc00b9eb9222ce3cabc8dc3386c917fa.boutiques",
    name="antsMotionCorrStats",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsMotionCorrStatsParamsDictNoTag = typing.TypedDict('_AntsMotionCorrStatsParamsDictNoTag', {
    "mask": InputPathType,
    "moco_params": InputPathType,
    "output": str,
    "transform_index": typing.NotRequired[int | None],
    "framewise": typing.NotRequired[bool | None],
    "spatial_map": bool,
    "timeseries_displacement": bool,
    "help": typing.NotRequired[bool | None],
})
AntsMotionCorrStatsParamsDictTagged = typing.TypedDict('AntsMotionCorrStatsParamsDictTagged', {
    "@type": typing.Literal["ants/antsMotionCorrStats"],
    "mask": InputPathType,
    "moco_params": InputPathType,
    "output": str,
    "transform_index": typing.NotRequired[int | None],
    "framewise": typing.NotRequired[bool | None],
    "spatial_map": bool,
    "timeseries_displacement": bool,
    "help": typing.NotRequired[bool | None],
})
AntsMotionCorrStatsParamsDict = _AntsMotionCorrStatsParamsDictNoTag | AntsMotionCorrStatsParamsDictTagged


class AntsMotionCorrStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsMotionCorrStatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    corrected_csv: OutputPathType
    """The corrected motion parameters.csv file."""


def ants_motion_corr_stats_params(
    mask: InputPathType,
    moco_params: InputPathType,
    output: str,
    transform_index: int | None = None,
    framewise: bool | None = None,
    spatial_map: bool = False,
    timeseries_displacement: bool = False,
    help_: bool | None = None,
) -> AntsMotionCorrStatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        mask: Mask image - compute displacements within mask.
        moco_params: Motion correction parameters from antsMotionCorr.
        output: Specify the output file.
        transform_index: Specify the index for a 3D transform to output.
        framewise: Do framewise summary stats.
        spatial_map: Output image of displacement magnitude.
        timeseries_displacement: Output 4d time-series image of displacement\
            magnitude.
        help_: Print the help menu. Short version with -h.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsMotionCorrStats",
        "mask": mask,
        "moco_params": moco_params,
        "output": output,
        "spatial_map": spatial_map,
        "timeseries_displacement": timeseries_displacement,
    }
    if transform_index is not None:
        params["transform_index"] = transform_index
    if framewise is not None:
        params["framewise"] = framewise
    if help_ is not None:
        params["help"] = help_
    return params


def ants_motion_corr_stats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsMotionCorrStatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mask", None) is None:
        raise StyxValidationError("`mask` must not be None")
    if not isinstance(params["mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType`')
    if params.get("moco_params", None) is None:
        raise StyxValidationError("`moco_params` must not be None")
    if not isinstance(params["moco_params"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moco_params` has the wrong type: Received `{type(params.get("moco_params", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("transform_index", None) is not None:
        if not isinstance(params["transform_index"], int):
            raise StyxValidationError(f'`transform_index` has the wrong type: Received `{type(params.get("transform_index", None))}` expected `int | None`')
    if params.get("framewise", None) is not None:
        if not isinstance(params["framewise"], bool):
            raise StyxValidationError(f'`framewise` has the wrong type: Received `{type(params.get("framewise", None))}` expected `bool | None`')
    if params.get("spatial_map", False) is None:
        raise StyxValidationError("`spatial_map` must not be None")
    if not isinstance(params["spatial_map"], bool):
        raise StyxValidationError(f'`spatial_map` has the wrong type: Received `{type(params.get("spatial_map", False))}` expected `bool`')
    if params.get("timeseries_displacement", False) is None:
        raise StyxValidationError("`timeseries_displacement` must not be None")
    if not isinstance(params["timeseries_displacement"], bool):
        raise StyxValidationError(f'`timeseries_displacement` has the wrong type: Received `{type(params.get("timeseries_displacement", False))}` expected `bool`')
    if params.get("help", None) is not None:
        if not isinstance(params["help"], bool):
            raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", None))}` expected `bool | None`')


def ants_motion_corr_stats_cargs(
    params: AntsMotionCorrStatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsMotionCorrStats")
    cargs.extend([
        "-x",
        execution.input_file(params.get("mask", None))
    ])
    cargs.extend([
        "-m",
        execution.input_file(params.get("moco_params", None))
    ])
    cargs.extend([
        "-o",
        params.get("output", None)
    ])
    if params.get("transform_index", None) is not None:
        cargs.extend([
            "-t",
            str(params.get("transform_index", None))
        ])
    if params.get("framewise", None) is not None:
        cargs.extend([
            "-f",
            ("1" if params.get("framewise", None) else "0")
        ])
    if params.get("spatial_map", False):
        cargs.append("-s")
    if params.get("timeseries_displacement", False):
        cargs.append("-d")
    if params.get("help", None) is not None:
        cargs.extend([
            "--help",
            ("1" if params.get("help", None) else "0")
        ])
    return cargs


def ants_motion_corr_stats_outputs(
    params: AntsMotionCorrStatsParamsDict,
    execution: Execution,
) -> AntsMotionCorrStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsMotionCorrStatsOutputs(
        root=execution.output_file("."),
        corrected_csv=execution.output_file(params.get("output", None)),
    )
    return ret


def ants_motion_corr_stats_execute(
    params: AntsMotionCorrStatsParamsDict,
    runner: Runner | None = None,
) -> AntsMotionCorrStatsOutputs:
    """
    antsMotionCorrStats
    
    Create summary measures of the parameters that are output by antsMotionCorr.
    Currently only works for linear transforms. Outputs the mean and max
    displacements for the voxels within a provided mask, at each time point. By
    default the displacements are relative to the reference space, but the
    framewise option may be used to provide displacements between consecutive
    time points.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsMotionCorrStatsOutputs`).
    """
    ants_motion_corr_stats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_MOTION_CORR_STATS_METADATA)
    params = execution.params(params)
    cargs = ants_motion_corr_stats_cargs(params, execution)
    ret = ants_motion_corr_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_motion_corr_stats(
    mask: InputPathType,
    moco_params: InputPathType,
    output: str,
    transform_index: int | None = None,
    framewise: bool | None = None,
    spatial_map: bool = False,
    timeseries_displacement: bool = False,
    help_: bool | None = None,
    runner: Runner | None = None,
) -> AntsMotionCorrStatsOutputs:
    """
    antsMotionCorrStats
    
    Create summary measures of the parameters that are output by antsMotionCorr.
    Currently only works for linear transforms. Outputs the mean and max
    displacements for the voxels within a provided mask, at each time point. By
    default the displacements are relative to the reference space, but the
    framewise option may be used to provide displacements between consecutive
    time points.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        mask: Mask image - compute displacements within mask.
        moco_params: Motion correction parameters from antsMotionCorr.
        output: Specify the output file.
        transform_index: Specify the index for a 3D transform to output.
        framewise: Do framewise summary stats.
        spatial_map: Output image of displacement magnitude.
        timeseries_displacement: Output 4d time-series image of displacement\
            magnitude.
        help_: Print the help menu. Short version with -h.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsMotionCorrStatsOutputs`).
    """
    params = ants_motion_corr_stats_params(
        mask=mask,
        moco_params=moco_params,
        output=output,
        transform_index=transform_index,
        framewise=framewise,
        spatial_map=spatial_map,
        timeseries_displacement=timeseries_displacement,
        help_=help_,
    )
    return ants_motion_corr_stats_execute(params, runner)


__all__ = [
    "ANTS_MOTION_CORR_STATS_METADATA",
    "AntsMotionCorrStatsOutputs",
    "AntsMotionCorrStatsParamsDict",
    "AntsMotionCorrStatsParamsDictTagged",
    "ants_motion_corr_stats",
    "ants_motion_corr_stats_execute",
    "ants_motion_corr_stats_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

N4_BIAS_FIELD_CORRECTION_METADATA = Metadata(
    id="3f10828e86e53eef2772983bf2e0a920246c4729.boutiques",
    name="N4BiasFieldCorrection",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_N4BiasFieldCorrectionConvergenceParamsDictNoTag = typing.TypedDict('_N4BiasFieldCorrectionConvergenceParamsDictNoTag', {
    "convergence": list[int],
    "convergence_threshold": typing.NotRequired[float | None],
})
N4BiasFieldCorrectionConvergenceParamsDictTagged = typing.TypedDict('N4BiasFieldCorrectionConvergenceParamsDictTagged', {
    "@type": typing.Literal["convergence"],
    "convergence": list[int],
    "convergence_threshold": typing.NotRequired[float | None],
})
N4BiasFieldCorrectionConvergenceParamsDict = _N4BiasFieldCorrectionConvergenceParamsDictNoTag | N4BiasFieldCorrectionConvergenceParamsDictTagged


_N4BiasFieldCorrectionBsplineFittingParamsDictNoTag = typing.TypedDict('_N4BiasFieldCorrectionBsplineFittingParamsDictNoTag', {
    "spline_distance": list[float],
    "spline_order": typing.NotRequired[int | None],
})
N4BiasFieldCorrectionBsplineFittingParamsDictTagged = typing.TypedDict('N4BiasFieldCorrectionBsplineFittingParamsDictTagged', {
    "@type": typing.Literal["bspline_fitting"],
    "spline_distance": list[float],
    "spline_order": typing.NotRequired[int | None],
})
N4BiasFieldCorrectionBsplineFittingParamsDict = _N4BiasFieldCorrectionBsplineFittingParamsDictNoTag | N4BiasFieldCorrectionBsplineFittingParamsDictTagged


_N4BiasFieldCorrectionHistogramSharpeningParamsDictNoTag = typing.TypedDict('_N4BiasFieldCorrectionHistogramSharpeningParamsDictNoTag', {
    "fwhm": typing.NotRequired[float | None],
    "wiener_noise": typing.NotRequired[float | None],
    "number_of_histogram_bins": typing.NotRequired[int | None],
})
N4BiasFieldCorrectionHistogramSharpeningParamsDictTagged = typing.TypedDict('N4BiasFieldCorrectionHistogramSharpeningParamsDictTagged', {
    "@type": typing.Literal["histogram_sharpening"],
    "fwhm": typing.NotRequired[float | None],
    "wiener_noise": typing.NotRequired[float | None],
    "number_of_histogram_bins": typing.NotRequired[int | None],
})
N4BiasFieldCorrectionHistogramSharpeningParamsDict = _N4BiasFieldCorrectionHistogramSharpeningParamsDictNoTag | N4BiasFieldCorrectionHistogramSharpeningParamsDictTagged


_N4BiasFieldCorrectionCorrectedOutputParamsDictNoTag = typing.TypedDict('_N4BiasFieldCorrectionCorrectedOutputParamsDictNoTag', {
    "correctedOutputFileName": str,
})
N4BiasFieldCorrectionCorrectedOutputParamsDictTagged = typing.TypedDict('N4BiasFieldCorrectionCorrectedOutputParamsDictTagged', {
    "@type": typing.Literal["correctedOutput"],
    "correctedOutputFileName": str,
})
N4BiasFieldCorrectionCorrectedOutputParamsDict = _N4BiasFieldCorrectionCorrectedOutputParamsDictNoTag | N4BiasFieldCorrectionCorrectedOutputParamsDictTagged


_N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictNoTag = typing.TypedDict('_N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictNoTag', {
    "correctedOutputFileName": str,
    "biasFile": typing.NotRequired[str | None],
})
N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged = typing.TypedDict('N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged', {
    "@type": typing.Literal["correctedOutputNoise"],
    "correctedOutputFileName": str,
    "biasFile": typing.NotRequired[str | None],
})
N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict = _N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictNoTag | N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged


_N4BiasFieldCorrectionParamsDictNoTag = typing.TypedDict('_N4BiasFieldCorrectionParamsDictNoTag', {
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "shrink_factor": typing.NotRequired[int | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "rescale_intensities": typing.NotRequired[bool | None],
    "weight_image": typing.NotRequired[InputPathType | None],
    "convergence": typing.NotRequired[N4BiasFieldCorrectionConvergenceParamsDict | None],
    "bspline_fitting": typing.NotRequired[N4BiasFieldCorrectionBsplineFittingParamsDict | None],
    "histogram_sharpening": typing.NotRequired[N4BiasFieldCorrectionHistogramSharpeningParamsDict | None],
    "verbose": typing.NotRequired[bool | None],
    "input_image": InputPathType,
    "output": typing.Union[N4BiasFieldCorrectionCorrectedOutputParamsDictTagged, N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged],
})
N4BiasFieldCorrectionParamsDictTagged = typing.TypedDict('N4BiasFieldCorrectionParamsDictTagged', {
    "@type": typing.Literal["ants/N4BiasFieldCorrection"],
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "shrink_factor": typing.NotRequired[int | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "rescale_intensities": typing.NotRequired[bool | None],
    "weight_image": typing.NotRequired[InputPathType | None],
    "convergence": typing.NotRequired[N4BiasFieldCorrectionConvergenceParamsDict | None],
    "bspline_fitting": typing.NotRequired[N4BiasFieldCorrectionBsplineFittingParamsDict | None],
    "histogram_sharpening": typing.NotRequired[N4BiasFieldCorrectionHistogramSharpeningParamsDict | None],
    "verbose": typing.NotRequired[bool | None],
    "input_image": InputPathType,
    "output": typing.Union[N4BiasFieldCorrectionCorrectedOutputParamsDictTagged, N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged],
})
N4BiasFieldCorrectionParamsDict = _N4BiasFieldCorrectionParamsDictNoTag | N4BiasFieldCorrectionParamsDictTagged


def n4_bias_field_correction_output_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "correctedOutput": n4_bias_field_correction_corrected_output_cargs,
        "correctedOutputNoise": n4_bias_field_correction_corrected_output_noise_cargs,
    }.get(t)


def n4_bias_field_correction_output_outputs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "correctedOutput": n4_bias_field_correction_corrected_output_outputs,
        "correctedOutputNoise": n4_bias_field_correction_corrected_output_noise_outputs,
    }.get(t)


def n4_bias_field_correction_output_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "correctedOutput": n4_bias_field_correction_corrected_output_validate,
        "correctedOutputNoise": n4_bias_field_correction_corrected_output_noise_validate,
    }.get(t)


def n4_bias_field_correction_convergence(
    convergence: list[int],
    convergence_threshold: float | None = None,
) -> N4BiasFieldCorrectionConvergenceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        convergence:.
        convergence_threshold:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "convergence",
        "convergence": convergence,
    }
    if convergence_threshold is not None:
        params["convergence_threshold"] = convergence_threshold
    return params


def n4_bias_field_correction_convergence_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N4BiasFieldCorrectionConvergenceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("convergence", None) is None:
        raise StyxValidationError("`convergence` must not be None")
    if not isinstance(params["convergence"], list):
        raise StyxValidationError(f'`convergence` has the wrong type: Received `{type(params.get("convergence", None))}` expected `list[int]`')
    for e in params["convergence"]:
        if not isinstance(e, int):
            raise StyxValidationError(f'`convergence` has the wrong type: Received `{type(params.get("convergence", None))}` expected `list[int]`')
    if params.get("convergence_threshold", None) is not None:
        if not isinstance(params["convergence_threshold"], (float, int)):
            raise StyxValidationError(f'`convergence_threshold` has the wrong type: Received `{type(params.get("convergence_threshold", None))}` expected `float | None`')


def n4_bias_field_correction_convergence_cargs(
    params: N4BiasFieldCorrectionConvergenceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + "x".join(map(str, params.get("convergence", None))) + "," + (str(params.get("convergence_threshold", None)) if (params.get("convergence_threshold", None) is not None) else "") + "]")
    return cargs


def n4_bias_field_correction_bspline_fitting(
    spline_distance: list[float],
    spline_order: int | None = None,
) -> N4BiasFieldCorrectionBsplineFittingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        spline_distance:.
        spline_order:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "bspline_fitting",
        "spline_distance": spline_distance,
    }
    if spline_order is not None:
        params["spline_order"] = spline_order
    return params


def n4_bias_field_correction_bspline_fitting_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N4BiasFieldCorrectionBsplineFittingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("spline_distance", None) is None:
        raise StyxValidationError("`spline_distance` must not be None")
    if not isinstance(params["spline_distance"], list):
        raise StyxValidationError(f'`spline_distance` has the wrong type: Received `{type(params.get("spline_distance", None))}` expected `list[float]`')
    for e in params["spline_distance"]:
        if not isinstance(e, (float, int)):
            raise StyxValidationError(f'`spline_distance` has the wrong type: Received `{type(params.get("spline_distance", None))}` expected `list[float]`')
    if params.get("spline_order", None) is not None:
        if not isinstance(params["spline_order"], int):
            raise StyxValidationError(f'`spline_order` has the wrong type: Received `{type(params.get("spline_order", None))}` expected `int | None`')


def n4_bias_field_correction_bspline_fitting_cargs(
    params: N4BiasFieldCorrectionBsplineFittingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + "x".join(map(str, params.get("spline_distance", None))) + "," + (str(params.get("spline_order", None)) if (params.get("spline_order", None) is not None) else "") + "]")
    return cargs


def n4_bias_field_correction_histogram_sharpening(
    fwhm: float | None = None,
    wiener_noise: float | None = None,
    number_of_histogram_bins: int | None = None,
) -> N4BiasFieldCorrectionHistogramSharpeningParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fwhm:.
        wiener_noise:.
        number_of_histogram_bins:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "histogram_sharpening",
    }
    if fwhm is not None:
        params["fwhm"] = fwhm
    if wiener_noise is not None:
        params["wiener_noise"] = wiener_noise
    if number_of_histogram_bins is not None:
        params["number_of_histogram_bins"] = number_of_histogram_bins
    return params


def n4_bias_field_correction_histogram_sharpening_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N4BiasFieldCorrectionHistogramSharpeningParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("fwhm", None) is not None:
        if not isinstance(params["fwhm"], (float, int)):
            raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", None))}` expected `float | None`')
    if params.get("wiener_noise", None) is not None:
        if not isinstance(params["wiener_noise"], (float, int)):
            raise StyxValidationError(f'`wiener_noise` has the wrong type: Received `{type(params.get("wiener_noise", None))}` expected `float | None`')
    if params.get("number_of_histogram_bins", None) is not None:
        if not isinstance(params["number_of_histogram_bins"], int):
            raise StyxValidationError(f'`number_of_histogram_bins` has the wrong type: Received `{type(params.get("number_of_histogram_bins", None))}` expected `int | None`')


def n4_bias_field_correction_histogram_sharpening_cargs(
    params: N4BiasFieldCorrectionHistogramSharpeningParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("fwhm", None) is not None or params.get("wiener_noise", None) is not None or params.get("number_of_histogram_bins", None) is not None:
        cargs.append("[" + (str(params.get("fwhm", None)) if (params.get("fwhm", None) is not None) else "") + "," + (str(params.get("wiener_noise", None)) if (params.get("wiener_noise", None) is not None) else "") + "," + (str(params.get("number_of_histogram_bins", None)) if (params.get("number_of_histogram_bins", None) is not None) else "") + "]")
    return cargs


class N4BiasFieldCorrectionCorrectedOutputOutputs(typing.NamedTuple):
    """
    Output object returned when calling `N4BiasFieldCorrectionCorrectedOutputParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_outfile: OutputPathType
    """Bias corrected image."""


def n4_bias_field_correction_corrected_output(
    corrected_output_file_name: str,
) -> N4BiasFieldCorrectionCorrectedOutputParamsDictTagged:
    """
    Build parameters.
    
    Args:
        corrected_output_file_name: Output file name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "correctedOutput",
        "correctedOutputFileName": corrected_output_file_name,
    }
    return params


def n4_bias_field_correction_corrected_output_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N4BiasFieldCorrectionCorrectedOutputParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("correctedOutputFileName", None) is None:
        raise StyxValidationError("`correctedOutputFileName` must not be None")
    if not isinstance(params["correctedOutputFileName"], str):
        raise StyxValidationError(f'`correctedOutputFileName` has the wrong type: Received `{type(params.get("correctedOutputFileName", None))}` expected `str`')


def n4_bias_field_correction_corrected_output_cargs(
    params: N4BiasFieldCorrectionCorrectedOutputParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("correctedOutputFileName", None))
    return cargs


def n4_bias_field_correction_corrected_output_outputs(
    params: N4BiasFieldCorrectionCorrectedOutputParamsDict,
    execution: Execution,
) -> N4BiasFieldCorrectionCorrectedOutputOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = N4BiasFieldCorrectionCorrectedOutputOutputs(
        root=execution.output_file("."),
        output_image_outfile=execution.output_file(params.get("correctedOutputFileName", None)),
    )
    return ret


class N4BiasFieldCorrectionCorrectedOutputNoiseOutputs(typing.NamedTuple):
    """
    Output object returned when calling `N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_outfile: OutputPathType
    """Bias corrected image."""
    output_bias_image: OutputPathType | None
    """Bias field image."""


def n4_bias_field_correction_corrected_output_noise(
    corrected_output_file_name: str,
    bias_file: str | None = None,
) -> N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged:
    """
    Build parameters.
    
    Args:
        corrected_output_file_name: Output file name.
        bias_file: Output bias field image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "correctedOutputNoise",
        "correctedOutputFileName": corrected_output_file_name,
    }
    if bias_file is not None:
        params["biasFile"] = bias_file
    return params


def n4_bias_field_correction_corrected_output_noise_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("correctedOutputFileName", None) is None:
        raise StyxValidationError("`correctedOutputFileName` must not be None")
    if not isinstance(params["correctedOutputFileName"], str):
        raise StyxValidationError(f'`correctedOutputFileName` has the wrong type: Received `{type(params.get("correctedOutputFileName", None))}` expected `str`')
    if params.get("biasFile", None) is not None:
        if not isinstance(params["biasFile"], str):
            raise StyxValidationError(f'`biasFile` has the wrong type: Received `{type(params.get("biasFile", None))}` expected `str | None`')


def n4_bias_field_correction_corrected_output_noise_cargs(
    params: N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + params.get("correctedOutputFileName", None) + "," + (params.get("biasFile", None) if (params.get("biasFile", None) is not None) else "") + "]")
    return cargs


def n4_bias_field_correction_corrected_output_noise_outputs(
    params: N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict,
    execution: Execution,
) -> N4BiasFieldCorrectionCorrectedOutputNoiseOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = N4BiasFieldCorrectionCorrectedOutputNoiseOutputs(
        root=execution.output_file("."),
        output_image_outfile=execution.output_file(params.get("correctedOutputFileName", None)),
        output_bias_image=execution.output_file(params.get("biasFile", None)) if (params.get("biasFile") is not None) else None,
    )
    return ret


class N4BiasFieldCorrectionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `N4BiasFieldCorrectionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: typing.Union[N4BiasFieldCorrectionCorrectedOutputOutputs, N4BiasFieldCorrectionCorrectedOutputNoiseOutputs]
    """Outputs from `N4BiasFieldCorrectionCorrectedOutputParamsDict` or
    `N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict`."""


def n4_bias_field_correction_params(
    input_image: InputPathType,
    output: typing.Union[N4BiasFieldCorrectionCorrectedOutputParamsDictTagged, N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged],
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    shrink_factor: int | None = None,
    mask_image: InputPathType | None = None,
    rescale_intensities: bool | None = None,
    weight_image: InputPathType | None = None,
    convergence: N4BiasFieldCorrectionConvergenceParamsDict | None = None,
    bspline_fitting: N4BiasFieldCorrectionBsplineFittingParamsDict | None = None,
    histogram_sharpening: N4BiasFieldCorrectionHistogramSharpeningParamsDict | None = None,
    verbose: bool | None = None,
) -> N4BiasFieldCorrectionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_image: -i, --input-image inputImageFilename. A scalar image is\
            expected as input for bias correction. Since N4 log transforms the\
            intensities, negative values or values close to zero should be\
            processed prior to correction.
        output: The bias corrected version of the input image, with optional\
            noise image.
        image_dimensionality: -d, --image-dimensionality 2/3/4. This option\
            forces the image to be treated as a specified-dimensional image. If not\
            specified, N4 tries to infer the dimensionality from the input image.
        shrink_factor: -s, --shrink-factor 1/2/3/(4)/... Running N4 on large\
            images can be time consuming. To lessen computation time, the input\
            image can be resampled. The shrink factor, specified as a single\
            integer, describes this resampling. Shrink factors <= 4 are commonly\
            used. Note that the shrink factor is only applied to the first two or\
            three dimensions which we assume are spatial.
        mask_image: -x, --mask-image maskImageFilename. If a mask image is\
            specified, the final bias correction is only performed in the mask\
            region. If a weight image is not specified, only intensity values\
            inside the masked region are used during the execution of the\
            algorithm. If a weight image is specified, only the non-zero weights\
            are used in the execution of the algorithm although the mask region\
            defines where bias correction is performed in the final output.\
            Otherwise bias correction occurs over the entire image domain. See also\
            the option description for the weight image. If a mask image is *not*\
            specified then the entire image region will be used as the mask region.\
            Note that this is different than the N3 implementation which uses the\
            results of Otsu thresholding to define a mask. However, this leads to\
            unknown anatomical regions being included and excluded during the bias\
            correction.
        rescale_intensities: -r, --rescale-intensities 0/(1). At each\
            iteration, a new intensity mapping is calculated and applied but there\
            is nothing which constrains the new intensity range to be within\
            certain values. The result is that the range can "drift" from the\
            original at each iteration. This option rescales to the [min,max] range\
            of the original image intensities within the user-specified mask. A\
            mask is required to perform rescaling.
        weight_image: -w, --weight-image weightImageFilename. The weight image\
            allows the user to perform a relative weighting of specific voxels\
            during the B-spline fitting. For example, some studies have shown that\
            N3 performed on white matter segmentations improves performance. If one\
            has a spatial probability map of the white matter, one can use this map\
            to weight the b-spline fitting towards those voxels which are more\
            probabilistically classified as white matter. See also the option\
            description for the mask image.
        convergence: -c, --convergence\
            [<numberOfIterations=50x50x50x50>,<convergenceThreshold=0.0>].\
            Convergence is determined by calculating the coefficient of variation\
            between subsequent iterations. When this value is less than the\
            specified threshold from the previous iteration or the maximum number\
            of iterations is exceeded the program terminates. Multiple resolutions\
            can be specified by using 'x' between the number of iterations at each\
            resolution, e.g. 100x50x50.
        bspline_fitting: -b, --bspline-fitting\
            [splineDistance,<splineOrder=3>]. These options describe the b-spline\
            fitting parameters. The initial b-spline mesh at the coarsest\
            resolution is specified either as the number of elements in each\
            dimension, e.g. 2x2x3 for 3-D images, or it can be specified as a\
            single scalar parameter which describes the isotropic sizing of the\
            mesh elements. The latter option is typically preferred. For each\
            subsequent level, the spline distance decreases in half, or\
            equivalently, the number of mesh elements doubles Cubic splines (order\
            = 3) are typically used. The default setting is to employ a single mesh\
            element over the entire domain, i.e., -b [1x1x1,3].
        histogram_sharpening: -t, --histogram-sharpening\
            [<FWHM=0.15>,<wienerNoise=0.01>,<numberOfHistogramBins=200>]. These\
            options describe the histogram sharpening parameters, i.e. the\
            deconvolution step parameters described in the original N3 algorithm.\
            The default values have been shown to work fairly well.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/N4BiasFieldCorrection",
        "input_image": input_image,
        "output": output,
    }
    if image_dimensionality is not None:
        params["image_dimensionality"] = image_dimensionality
    if shrink_factor is not None:
        params["shrink_factor"] = shrink_factor
    if mask_image is not None:
        params["mask_image"] = mask_image
    if rescale_intensities is not None:
        params["rescale_intensities"] = rescale_intensities
    if weight_image is not None:
        params["weight_image"] = weight_image
    if convergence is not None:
        params["convergence"] = convergence
    if bspline_fitting is not None:
        params["bspline_fitting"] = bspline_fitting
    if histogram_sharpening is not None:
        params["histogram_sharpening"] = histogram_sharpening
    if verbose is not None:
        params["verbose"] = verbose
    return params


def n4_bias_field_correction_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N4BiasFieldCorrectionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimensionality", None) is not None:
        if not isinstance(params["image_dimensionality"], int):
            raise StyxValidationError(f'`image_dimensionality` has the wrong type: Received `{type(params.get("image_dimensionality", None))}` expected `typing.Literal[2, 3, 4] | None`')
        if params["image_dimensionality"] not in [2, 3, 4]:
            raise StyxValidationError("Parameter `image_dimensionality` must be one of [2, 3, 4]")
    if params.get("shrink_factor", None) is not None:
        if not isinstance(params["shrink_factor"], int):
            raise StyxValidationError(f'`shrink_factor` has the wrong type: Received `{type(params.get("shrink_factor", None))}` expected `int | None`')
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("rescale_intensities", None) is not None:
        if not isinstance(params["rescale_intensities"], bool):
            raise StyxValidationError(f'`rescale_intensities` has the wrong type: Received `{type(params.get("rescale_intensities", None))}` expected `bool | None`')
    if params.get("weight_image", None) is not None:
        if not isinstance(params["weight_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`weight_image` has the wrong type: Received `{type(params.get("weight_image", None))}` expected `InputPathType | None`')
    if params.get("convergence", None) is not None:
        n4_bias_field_correction_convergence_validate(params["convergence"])
    if params.get("bspline_fitting", None) is not None:
        n4_bias_field_correction_bspline_fitting_validate(params["bspline_fitting"])
    if params.get("histogram_sharpening", None) is not None:
        n4_bias_field_correction_histogram_sharpening_validate(params["histogram_sharpening"])
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], bool):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `bool | None`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["output"])}\'')
    if "@type" not in params["output"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["output"]["@type"] not in ["correctedOutput", "correctedOutputNoise"]:
        raise StyxValidationError("Parameter `output`s `@type` must be one of [\"correctedOutput\", \"correctedOutputNoise\"]")
    n4_bias_field_correction_output_validate_dyn_fn(params["output"]["@type"])(params["output"])


def n4_bias_field_correction_cargs(
    params: N4BiasFieldCorrectionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("N4BiasFieldCorrection")
    if params.get("image_dimensionality", None) is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("image_dimensionality", None))
        ])
    if params.get("shrink_factor", None) is not None:
        cargs.extend([
            "--shrink-factor",
            str(params.get("shrink_factor", None))
        ])
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "--mask-image",
            execution.input_file(params.get("mask_image", None))
        ])
    if params.get("rescale_intensities", None) is not None:
        cargs.extend([
            "--rescale-intensities",
            ("1" if params.get("rescale_intensities", None) else "0")
        ])
    if params.get("weight_image", None) is not None:
        cargs.extend([
            "--weight-image",
            execution.input_file(params.get("weight_image", None))
        ])
    if params.get("convergence", None) is not None:
        cargs.extend([
            "--convergence",
            *n4_bias_field_correction_convergence_cargs(params.get("convergence", None), execution)
        ])
    if params.get("bspline_fitting", None) is not None:
        cargs.extend([
            "--bspline-fitting",
            *n4_bias_field_correction_bspline_fitting_cargs(params.get("bspline_fitting", None), execution)
        ])
    if params.get("histogram_sharpening", None) is not None:
        cargs.extend([
            "--histogram-sharpening",
            *n4_bias_field_correction_histogram_sharpening_cargs(params.get("histogram_sharpening", None), execution)
        ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "--verbose",
            ("1" if params.get("verbose", None) else "0")
        ])
    cargs.extend([
        "--input-image",
        execution.input_file(params.get("input_image", None))
    ])
    cargs.extend([
        "--output",
        *n4_bias_field_correction_output_cargs_dyn_fn(params.get("output", None)["@type"])(params.get("output", None), execution)
    ])
    return cargs


def n4_bias_field_correction_outputs(
    params: N4BiasFieldCorrectionParamsDict,
    execution: Execution,
) -> N4BiasFieldCorrectionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = N4BiasFieldCorrectionOutputs(
        root=execution.output_file("."),
        output=n4_bias_field_correction_output_outputs_dyn_fn(params.get("output")["@type"])(params.get("output"), execution),
    )
    return ret


def n4_bias_field_correction_execute(
    params: N4BiasFieldCorrectionParamsDict,
    runner: Runner | None = None,
) -> N4BiasFieldCorrectionOutputs:
    """
    N4BiasFieldCorrection
    
    N4 is a variant of the popular N3 (nonparameteric nonuniform normalization)
    retrospective bias correction algorithm. Based on the assumption that the
    corruption of the low frequency bias field can be modeled as a convolution
    of the intensity histogram by a Gaussian, the basic algorithmic protocol is
    to iterate between deconvolving the intensity histogram by a Gaussian,
    remapping the intensities, and then spatially smoothing this result by a
    B-spline modeling of the bias field itself.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `N4BiasFieldCorrectionOutputs`).
    """
    n4_bias_field_correction_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(N4_BIAS_FIELD_CORRECTION_METADATA)
    params = execution.params(params)
    cargs = n4_bias_field_correction_cargs(params, execution)
    ret = n4_bias_field_correction_outputs(params, execution)
    execution.run(cargs)
    return ret


def n4_bias_field_correction(
    input_image: InputPathType,
    output: typing.Union[N4BiasFieldCorrectionCorrectedOutputParamsDictTagged, N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged],
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    shrink_factor: int | None = None,
    mask_image: InputPathType | None = None,
    rescale_intensities: bool | None = None,
    weight_image: InputPathType | None = None,
    convergence: N4BiasFieldCorrectionConvergenceParamsDict | None = None,
    bspline_fitting: N4BiasFieldCorrectionBsplineFittingParamsDict | None = None,
    histogram_sharpening: N4BiasFieldCorrectionHistogramSharpeningParamsDict | None = None,
    verbose: bool | None = None,
    runner: Runner | None = None,
) -> N4BiasFieldCorrectionOutputs:
    """
    N4BiasFieldCorrection
    
    N4 is a variant of the popular N3 (nonparameteric nonuniform normalization)
    retrospective bias correction algorithm. Based on the assumption that the
    corruption of the low frequency bias field can be modeled as a convolution
    of the intensity histogram by a Gaussian, the basic algorithmic protocol is
    to iterate between deconvolving the intensity histogram by a Gaussian,
    remapping the intensities, and then spatially smoothing this result by a
    B-spline modeling of the bias field itself.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_image: -i, --input-image inputImageFilename. A scalar image is\
            expected as input for bias correction. Since N4 log transforms the\
            intensities, negative values or values close to zero should be\
            processed prior to correction.
        output: The bias corrected version of the input image, with optional\
            noise image.
        image_dimensionality: -d, --image-dimensionality 2/3/4. This option\
            forces the image to be treated as a specified-dimensional image. If not\
            specified, N4 tries to infer the dimensionality from the input image.
        shrink_factor: -s, --shrink-factor 1/2/3/(4)/... Running N4 on large\
            images can be time consuming. To lessen computation time, the input\
            image can be resampled. The shrink factor, specified as a single\
            integer, describes this resampling. Shrink factors <= 4 are commonly\
            used. Note that the shrink factor is only applied to the first two or\
            three dimensions which we assume are spatial.
        mask_image: -x, --mask-image maskImageFilename. If a mask image is\
            specified, the final bias correction is only performed in the mask\
            region. If a weight image is not specified, only intensity values\
            inside the masked region are used during the execution of the\
            algorithm. If a weight image is specified, only the non-zero weights\
            are used in the execution of the algorithm although the mask region\
            defines where bias correction is performed in the final output.\
            Otherwise bias correction occurs over the entire image domain. See also\
            the option description for the weight image. If a mask image is *not*\
            specified then the entire image region will be used as the mask region.\
            Note that this is different than the N3 implementation which uses the\
            results of Otsu thresholding to define a mask. However, this leads to\
            unknown anatomical regions being included and excluded during the bias\
            correction.
        rescale_intensities: -r, --rescale-intensities 0/(1). At each\
            iteration, a new intensity mapping is calculated and applied but there\
            is nothing which constrains the new intensity range to be within\
            certain values. The result is that the range can "drift" from the\
            original at each iteration. This option rescales to the [min,max] range\
            of the original image intensities within the user-specified mask. A\
            mask is required to perform rescaling.
        weight_image: -w, --weight-image weightImageFilename. The weight image\
            allows the user to perform a relative weighting of specific voxels\
            during the B-spline fitting. For example, some studies have shown that\
            N3 performed on white matter segmentations improves performance. If one\
            has a spatial probability map of the white matter, one can use this map\
            to weight the b-spline fitting towards those voxels which are more\
            probabilistically classified as white matter. See also the option\
            description for the mask image.
        convergence: -c, --convergence\
            [<numberOfIterations=50x50x50x50>,<convergenceThreshold=0.0>].\
            Convergence is determined by calculating the coefficient of variation\
            between subsequent iterations. When this value is less than the\
            specified threshold from the previous iteration or the maximum number\
            of iterations is exceeded the program terminates. Multiple resolutions\
            can be specified by using 'x' between the number of iterations at each\
            resolution, e.g. 100x50x50.
        bspline_fitting: -b, --bspline-fitting\
            [splineDistance,<splineOrder=3>]. These options describe the b-spline\
            fitting parameters. The initial b-spline mesh at the coarsest\
            resolution is specified either as the number of elements in each\
            dimension, e.g. 2x2x3 for 3-D images, or it can be specified as a\
            single scalar parameter which describes the isotropic sizing of the\
            mesh elements. The latter option is typically preferred. For each\
            subsequent level, the spline distance decreases in half, or\
            equivalently, the number of mesh elements doubles Cubic splines (order\
            = 3) are typically used. The default setting is to employ a single mesh\
            element over the entire domain, i.e., -b [1x1x1,3].
        histogram_sharpening: -t, --histogram-sharpening\
            [<FWHM=0.15>,<wienerNoise=0.01>,<numberOfHistogramBins=200>]. These\
            options describe the histogram sharpening parameters, i.e. the\
            deconvolution step parameters described in the original N3 algorithm.\
            The default values have been shown to work fairly well.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `N4BiasFieldCorrectionOutputs`).
    """
    params = n4_bias_field_correction_params(
        image_dimensionality=image_dimensionality,
        shrink_factor=shrink_factor,
        mask_image=mask_image,
        rescale_intensities=rescale_intensities,
        weight_image=weight_image,
        convergence=convergence,
        bspline_fitting=bspline_fitting,
        histogram_sharpening=histogram_sharpening,
        verbose=verbose,
        input_image=input_image,
        output=output,
    )
    return n4_bias_field_correction_execute(params, runner)


__all__ = [
    "N4BiasFieldCorrectionBsplineFittingParamsDict",
    "N4BiasFieldCorrectionBsplineFittingParamsDictTagged",
    "N4BiasFieldCorrectionConvergenceParamsDict",
    "N4BiasFieldCorrectionConvergenceParamsDictTagged",
    "N4BiasFieldCorrectionCorrectedOutputNoiseOutputs",
    "N4BiasFieldCorrectionCorrectedOutputNoiseParamsDict",
    "N4BiasFieldCorrectionCorrectedOutputNoiseParamsDictTagged",
    "N4BiasFieldCorrectionCorrectedOutputOutputs",
    "N4BiasFieldCorrectionCorrectedOutputParamsDict",
    "N4BiasFieldCorrectionCorrectedOutputParamsDictTagged",
    "N4BiasFieldCorrectionHistogramSharpeningParamsDict",
    "N4BiasFieldCorrectionHistogramSharpeningParamsDictTagged",
    "N4BiasFieldCorrectionOutputs",
    "N4BiasFieldCorrectionParamsDict",
    "N4BiasFieldCorrectionParamsDictTagged",
    "N4_BIAS_FIELD_CORRECTION_METADATA",
    "n4_bias_field_correction",
    "n4_bias_field_correction_bspline_fitting",
    "n4_bias_field_correction_convergence",
    "n4_bias_field_correction_corrected_output",
    "n4_bias_field_correction_corrected_output_noise",
    "n4_bias_field_correction_execute",
    "n4_bias_field_correction_histogram_sharpening",
    "n4_bias_field_correction_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

PASTE_IMAGE_INTO_IMAGE_METADATA = Metadata(
    id="9434da4462a4b038646e9361e0a0659838206323.boutiques",
    name="PasteImageIntoImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_PasteImageIntoImageParamsDictNoTag = typing.TypedDict('_PasteImageIntoImageParamsDictNoTag', {
    "image_dimension": int,
    "input_canvas_image": InputPathType,
    "input_image": InputPathType,
    "output_image": str,
    "start_index": str,
    "background_label": typing.NotRequired[int | None],
    "paint_over_non_background_voxels": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "conflict_label": typing.NotRequired[int | None],
})
PasteImageIntoImageParamsDictTagged = typing.TypedDict('PasteImageIntoImageParamsDictTagged', {
    "@type": typing.Literal["ants/PasteImageIntoImage"],
    "image_dimension": int,
    "input_canvas_image": InputPathType,
    "input_image": InputPathType,
    "output_image": str,
    "start_index": str,
    "background_label": typing.NotRequired[int | None],
    "paint_over_non_background_voxels": typing.NotRequired[typing.Literal[0, 1, 2] | None],
    "conflict_label": typing.NotRequired[int | None],
})
PasteImageIntoImageParamsDict = _PasteImageIntoImageParamsDictNoTag | PasteImageIntoImageParamsDictTagged


class PasteImageIntoImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `PasteImageIntoImageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """The final output image with the input image pasted onto the canvas."""


def paste_image_into_image_params(
    image_dimension: int,
    input_canvas_image: InputPathType,
    input_image: InputPathType,
    output_image: str,
    start_index: str,
    background_label: int | None = None,
    paint_over_non_background_voxels: typing.Literal[0, 1, 2] | None = None,
    conflict_label: int | None = None,
) -> PasteImageIntoImageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Specify the dimension of the images.
        input_canvas_image: The canvas image on which the input image will be\
            pasted.
        input_image: The image to be pasted onto the canvas.
        output_image: The resulting image after pasting.
        start_index: The starting index where the input image will be pasted on\
            the canvas.
        background_label: The label value considered as background.
        paint_over_non_background_voxels: Defines behavior when the input image\
            voxel is non-background and the corresponding canvas voxel is\
            background: 0 - leave as is, 1 - replace with input voxel value, 2 -\
            replace with conflict label.
        conflict_label: The label value used for conflicting non-background\
            voxels if 'paintOverNonBackgroundVoxels' is set to 2.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/PasteImageIntoImage",
        "image_dimension": image_dimension,
        "input_canvas_image": input_canvas_image,
        "input_image": input_image,
        "output_image": output_image,
        "start_index": start_index,
    }
    if background_label is not None:
        params["background_label"] = background_label
    if paint_over_non_background_voxels is not None:
        params["paint_over_non_background_voxels"] = paint_over_non_background_voxels
    if conflict_label is not None:
        params["conflict_label"] = conflict_label
    return params


def paste_image_into_image_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `PasteImageIntoImageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("input_canvas_image", None) is None:
        raise StyxValidationError("`input_canvas_image` must not be None")
    if not isinstance(params["input_canvas_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_canvas_image` has the wrong type: Received `{type(params.get("input_canvas_image", None))}` expected `InputPathType`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("start_index", None) is None:
        raise StyxValidationError("`start_index` must not be None")
    if not isinstance(params["start_index"], str):
        raise StyxValidationError(f'`start_index` has the wrong type: Received `{type(params.get("start_index", None))}` expected `str`')
    if params.get("background_label", None) is not None:
        if not isinstance(params["background_label"], int):
            raise StyxValidationError(f'`background_label` has the wrong type: Received `{type(params.get("background_label", None))}` expected `int | None`')
    if params.get("paint_over_non_background_voxels", None) is not None:
        if not isinstance(params["paint_over_non_background_voxels"], int):
            raise StyxValidationError(f'`paint_over_non_background_voxels` has the wrong type: Received `{type(params.get("paint_over_non_background_voxels", None))}` expected `typing.Literal[0, 1, 2] | None`')
        if params["paint_over_non_background_voxels"] not in [0, 1, 2]:
            raise StyxValidationError("Parameter `paint_over_non_background_voxels` must be one of [0, 1, 2]")
    if params.get("conflict_label", None) is not None:
        if not isinstance(params["conflict_label"], int):
            raise StyxValidationError(f'`conflict_label` has the wrong type: Received `{type(params.get("conflict_label", None))}` expected `int | None`')


def paste_image_into_image_cargs(
    params: PasteImageIntoImageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("PasteImageIntoImage")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("input_canvas_image", None)))
    cargs.append(execution.input_file(params.get("input_image", None)))
    cargs.append(params.get("output_image", None))
    cargs.append(params.get("start_index", None))
    if params.get("background_label", None) is not None:
        cargs.append(str(params.get("background_label", None)))
    if params.get("paint_over_non_background_voxels", None) is not None:
        cargs.append(str(params.get("paint_over_non_background_voxels", None)))
    if params.get("conflict_label", None) is not None:
        cargs.append(str(params.get("conflict_label", None)))
    return cargs


def paste_image_into_image_outputs(
    params: PasteImageIntoImageParamsDict,
    execution: Execution,
) -> PasteImageIntoImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = PasteImageIntoImageOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(params.get("output_image", None)),
    )
    return ret


def paste_image_into_image_execute(
    params: PasteImageIntoImageParamsDict,
    runner: Runner | None = None,
) -> PasteImageIntoImageOutputs:
    """
    PasteImageIntoImage
    
    Paste the input image into the input canvas image. Depending on parameters,
    it can replace or merge existing voxel values.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `PasteImageIntoImageOutputs`).
    """
    paste_image_into_image_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(PASTE_IMAGE_INTO_IMAGE_METADATA)
    params = execution.params(params)
    cargs = paste_image_into_image_cargs(params, execution)
    ret = paste_image_into_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def paste_image_into_image(
    image_dimension: int,
    input_canvas_image: InputPathType,
    input_image: InputPathType,
    output_image: str,
    start_index: str,
    background_label: int | None = None,
    paint_over_non_background_voxels: typing.Literal[0, 1, 2] | None = None,
    conflict_label: int | None = None,
    runner: Runner | None = None,
) -> PasteImageIntoImageOutputs:
    """
    PasteImageIntoImage
    
    Paste the input image into the input canvas image. Depending on parameters,
    it can replace or merge existing voxel values.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Specify the dimension of the images.
        input_canvas_image: The canvas image on which the input image will be\
            pasted.
        input_image: The image to be pasted onto the canvas.
        output_image: The resulting image after pasting.
        start_index: The starting index where the input image will be pasted on\
            the canvas.
        background_label: The label value considered as background.
        paint_over_non_background_voxels: Defines behavior when the input image\
            voxel is non-background and the corresponding canvas voxel is\
            background: 0 - leave as is, 1 - replace with input voxel value, 2 -\
            replace with conflict label.
        conflict_label: The label value used for conflicting non-background\
            voxels if 'paintOverNonBackgroundVoxels' is set to 2.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `PasteImageIntoImageOutputs`).
    """
    params = paste_image_into_image_params(
        image_dimension=image_dimension,
        input_canvas_image=input_canvas_image,
        input_image=input_image,
        output_image=output_image,
        start_index=start_index,
        background_label=background_label,
        paint_over_non_background_voxels=paint_over_non_background_voxels,
        conflict_label=conflict_label,
    )
    return paste_image_into_image_execute(params, runner)


__all__ = [
    "PASTE_IMAGE_INTO_IMAGE_METADATA",
    "PasteImageIntoImageOutputs",
    "PasteImageIntoImageParamsDict",
    "PasteImageIntoImageParamsDictTagged",
    "paste_image_into_image",
    "paste_image_into_image_execute",
    "paste_image_into_image_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_SCALAR_IMAGE_TO_RGB_METADATA = Metadata(
    id="0188f81662be3b550786a9a91112208db37eb27b.boutiques",
    name="ConvertScalarImageToRGB",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_ConvertScalarImageToRgbParamsDictNoTag = typing.TypedDict('_ConvertScalarImageToRgbParamsDictNoTag', {
    "image_dimension": int,
    "input_image": InputPathType,
    "output_image": str,
    "mask": InputPathType,
    "colormap": typing.Literal["grey", "red", "green", "blue", "copper", "jet", "hsv", "spring", "summer", "autumn", "winter", "hot", "cool", "overunder", "custom"],
    "custom_colormap_file": typing.NotRequired[InputPathType | None],
    "minimum_input": typing.NotRequired[float | None],
    "maximum_input": typing.NotRequired[float | None],
    "minimum_rgb_output": typing.NotRequired[int | None],
    "maximum_rgb_output": typing.NotRequired[int | None],
    "vtk_lookup_table": typing.NotRequired[str | None],
})
ConvertScalarImageToRgbParamsDictTagged = typing.TypedDict('ConvertScalarImageToRgbParamsDictTagged', {
    "@type": typing.Literal["ants/ConvertScalarImageToRGB"],
    "image_dimension": int,
    "input_image": InputPathType,
    "output_image": str,
    "mask": InputPathType,
    "colormap": typing.Literal["grey", "red", "green", "blue", "copper", "jet", "hsv", "spring", "summer", "autumn", "winter", "hot", "cool", "overunder", "custom"],
    "custom_colormap_file": typing.NotRequired[InputPathType | None],
    "minimum_input": typing.NotRequired[float | None],
    "maximum_input": typing.NotRequired[float | None],
    "minimum_rgb_output": typing.NotRequired[int | None],
    "maximum_rgb_output": typing.NotRequired[int | None],
    "vtk_lookup_table": typing.NotRequired[str | None],
})
ConvertScalarImageToRgbParamsDict = _ConvertScalarImageToRgbParamsDictNoTag | ConvertScalarImageToRgbParamsDictTagged


class ConvertScalarImageToRgbOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertScalarImageToRgbParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_rgb_image: OutputPathType
    """The resulting RGB image after conversion."""


def convert_scalar_image_to_rgb_params(
    image_dimension: int,
    input_image: InputPathType,
    output_image: str,
    mask: InputPathType,
    colormap: typing.Literal["grey", "red", "green", "blue", "copper", "jet", "hsv", "spring", "summer", "autumn", "winter", "hot", "cool", "overunder", "custom"],
    custom_colormap_file: InputPathType | None = None,
    minimum_input: float | None = None,
    maximum_input: float | None = None,
    minimum_rgb_output: int | None = None,
    maximum_rgb_output: int | None = None,
    vtk_lookup_table: str | None = None,
) -> ConvertScalarImageToRgbParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the image (e.g., 2D or 3D).
        input_image: The input scalar image to be converted to RGB.
        output_image: The output RGB image file.
        mask: The mask image to apply during conversion.
        colormap: The colormap to use for RGB conversion.
        custom_colormap_file: The file specifying the custom colormap (only\
            used if colormap is 'custom').
        minimum_input: The minimum input value for scaling.
        maximum_input: The maximum input value for scaling.
        minimum_rgb_output: The minimum output value for the RGB image.\
            Defaults to 0.
        maximum_rgb_output: The maximum output value for the RGB image.\
            Defaults to 255.
        vtk_lookup_table: The VTK lookup table to apply for additional\
            customization.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ConvertScalarImageToRGB",
        "image_dimension": image_dimension,
        "input_image": input_image,
        "output_image": output_image,
        "mask": mask,
        "colormap": colormap,
    }
    if custom_colormap_file is not None:
        params["custom_colormap_file"] = custom_colormap_file
    if minimum_input is not None:
        params["minimum_input"] = minimum_input
    if maximum_input is not None:
        params["maximum_input"] = maximum_input
    if minimum_rgb_output is not None:
        params["minimum_rgb_output"] = minimum_rgb_output
    if maximum_rgb_output is not None:
        params["maximum_rgb_output"] = maximum_rgb_output
    if vtk_lookup_table is not None:
        params["vtk_lookup_table"] = vtk_lookup_table
    return params


def convert_scalar_image_to_rgb_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertScalarImageToRgbParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("mask", None) is None:
        raise StyxValidationError("`mask` must not be None")
    if not isinstance(params["mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType`')
    if params.get("colormap", None) is None:
        raise StyxValidationError("`colormap` must not be None")
    if not isinstance(params["colormap"], str):
        raise StyxValidationError(f'`colormap` has the wrong type: Received `{type(params.get("colormap", None))}` expected `typing.Literal["grey", "red", "green", "blue", "copper", "jet", "hsv", "spring", "summer", "autumn", "winter", "hot", "cool", "overunder", "custom"]`')
    if params["colormap"] not in ["grey", "red", "green", "blue", "copper", "jet", "hsv", "spring", "summer", "autumn", "winter", "hot", "cool", "overunder", "custom"]:
        raise StyxValidationError("Parameter `colormap` must be one of [\"grey\", \"red\", \"green\", \"blue\", \"copper\", \"jet\", \"hsv\", \"spring\", \"summer\", \"autumn\", \"winter\", \"hot\", \"cool\", \"overunder\", \"custom\"]")
    if params.get("custom_colormap_file", None) is not None:
        if not isinstance(params["custom_colormap_file"], (pathlib.Path, str)):
            raise StyxValidationError(f'`custom_colormap_file` has the wrong type: Received `{type(params.get("custom_colormap_file", None))}` expected `InputPathType | None`')
    if params.get("minimum_input", None) is not None:
        if not isinstance(params["minimum_input"], (float, int)):
            raise StyxValidationError(f'`minimum_input` has the wrong type: Received `{type(params.get("minimum_input", None))}` expected `float | None`')
    if params.get("maximum_input", None) is not None:
        if not isinstance(params["maximum_input"], (float, int)):
            raise StyxValidationError(f'`maximum_input` has the wrong type: Received `{type(params.get("maximum_input", None))}` expected `float | None`')
    if params.get("minimum_rgb_output", None) is not None:
        if not isinstance(params["minimum_rgb_output"], int):
            raise StyxValidationError(f'`minimum_rgb_output` has the wrong type: Received `{type(params.get("minimum_rgb_output", None))}` expected `int | None`')
    if params.get("maximum_rgb_output", None) is not None:
        if not isinstance(params["maximum_rgb_output"], int):
            raise StyxValidationError(f'`maximum_rgb_output` has the wrong type: Received `{type(params.get("maximum_rgb_output", None))}` expected `int | None`')
    if params.get("vtk_lookup_table", None) is not None:
        if not isinstance(params["vtk_lookup_table"], str):
            raise StyxValidationError(f'`vtk_lookup_table` has the wrong type: Received `{type(params.get("vtk_lookup_table", None))}` expected `str | None`')


def convert_scalar_image_to_rgb_cargs(
    params: ConvertScalarImageToRgbParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ConvertScalarImageToRGB")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("input_image", None)))
    cargs.append(params.get("output_image", None))
    cargs.append(execution.input_file(params.get("mask", None)))
    cargs.append(params.get("colormap", None))
    if params.get("custom_colormap_file", None) is not None:
        cargs.append(execution.input_file(params.get("custom_colormap_file", None)))
    if params.get("minimum_input", None) is not None:
        cargs.append(str(params.get("minimum_input", None)))
    if params.get("maximum_input", None) is not None:
        cargs.append(str(params.get("maximum_input", None)))
    if params.get("minimum_rgb_output", None) is not None:
        cargs.append(str(params.get("minimum_rgb_output", None)))
    if params.get("maximum_rgb_output", None) is not None:
        cargs.append(str(params.get("maximum_rgb_output", None)))
    if params.get("vtk_lookup_table", None) is not None:
        cargs.append(params.get("vtk_lookup_table", None))
    return cargs


def convert_scalar_image_to_rgb_outputs(
    params: ConvertScalarImageToRgbParamsDict,
    execution: Execution,
) -> ConvertScalarImageToRgbOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertScalarImageToRgbOutputs(
        root=execution.output_file("."),
        output_rgb_image=execution.output_file(params.get("output_image", None)),
    )
    return ret


def convert_scalar_image_to_rgb_execute(
    params: ConvertScalarImageToRgbParamsDict,
    runner: Runner | None = None,
) -> ConvertScalarImageToRgbOutputs:
    """
    ConvertScalarImageToRGB
    
    Converts a scalar image to an RGB image using specified parameters. Supports
    multiple colormap options and customization.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertScalarImageToRgbOutputs`).
    """
    convert_scalar_image_to_rgb_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_SCALAR_IMAGE_TO_RGB_METADATA)
    params = execution.params(params)
    cargs = convert_scalar_image_to_rgb_cargs(params, execution)
    ret = convert_scalar_image_to_rgb_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_scalar_image_to_rgb(
    image_dimension: int,
    input_image: InputPathType,
    output_image: str,
    mask: InputPathType,
    colormap: typing.Literal["grey", "red", "green", "blue", "copper", "jet", "hsv", "spring", "summer", "autumn", "winter", "hot", "cool", "overunder", "custom"],
    custom_colormap_file: InputPathType | None = None,
    minimum_input: float | None = None,
    maximum_input: float | None = None,
    minimum_rgb_output: int | None = None,
    maximum_rgb_output: int | None = None,
    vtk_lookup_table: str | None = None,
    runner: Runner | None = None,
) -> ConvertScalarImageToRgbOutputs:
    """
    ConvertScalarImageToRGB
    
    Converts a scalar image to an RGB image using specified parameters. Supports
    multiple colormap options and customization.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the image (e.g., 2D or 3D).
        input_image: The input scalar image to be converted to RGB.
        output_image: The output RGB image file.
        mask: The mask image to apply during conversion.
        colormap: The colormap to use for RGB conversion.
        custom_colormap_file: The file specifying the custom colormap (only\
            used if colormap is 'custom').
        minimum_input: The minimum input value for scaling.
        maximum_input: The maximum input value for scaling.
        minimum_rgb_output: The minimum output value for the RGB image.\
            Defaults to 0.
        maximum_rgb_output: The maximum output value for the RGB image.\
            Defaults to 255.
        vtk_lookup_table: The VTK lookup table to apply for additional\
            customization.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertScalarImageToRgbOutputs`).
    """
    params = convert_scalar_image_to_rgb_params(
        image_dimension=image_dimension,
        input_image=input_image,
        output_image=output_image,
        mask=mask,
        colormap=colormap,
        custom_colormap_file=custom_colormap_file,
        minimum_input=minimum_input,
        maximum_input=maximum_input,
        minimum_rgb_output=minimum_rgb_output,
        maximum_rgb_output=maximum_rgb_output,
        vtk_lookup_table=vtk_lookup_table,
    )
    return convert_scalar_image_to_rgb_execute(params, runner)


__all__ = [
    "CONVERT_SCALAR_IMAGE_TO_RGB_METADATA",
    "ConvertScalarImageToRgbOutputs",
    "ConvertScalarImageToRgbParamsDict",
    "ConvertScalarImageToRgbParamsDictTagged",
    "convert_scalar_image_to_rgb",
    "convert_scalar_image_to_rgb_execute",
    "convert_scalar_image_to_rgb_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA = Metadata(
    id="bf03b521899d77cb41c538f595ad6ed7f0d4440c.boutiques",
    name="ANTSUseLandmarkImagesToGetAffineTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsuseLandmarkImagesToGetAffineTransformParamsDictNoTag = typing.TypedDict('_AntsuseLandmarkImagesToGetAffineTransformParamsDictNoTag', {
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "transform_type": typing.Literal["rigid", "affine"],
    "output_affine": str,
})
AntsuseLandmarkImagesToGetAffineTransformParamsDictTagged = typing.TypedDict('AntsuseLandmarkImagesToGetAffineTransformParamsDictTagged', {
    "@type": typing.Literal["ants/ANTSUseLandmarkImagesToGetAffineTransform"],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "transform_type": typing.Literal["rigid", "affine"],
    "output_affine": str,
})
AntsuseLandmarkImagesToGetAffineTransformParamsDict = _AntsuseLandmarkImagesToGetAffineTransformParamsDictNoTag | AntsuseLandmarkImagesToGetAffineTransformParamsDictTagged


class AntsuseLandmarkImagesToGetAffineTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsuseLandmarkImagesToGetAffineTransformParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    affine_transform_matrix: OutputPathType
    """The output is the affine transformation matrix file."""


def antsuse_landmark_images_to_get_affine_transform_params(
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
) -> AntsuseLandmarkImagesToGetAffineTransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fixed_image: The fixed image containing labeled landmarks (N-ary\
            image).
        moving_image: The moving image containing labeled landmarks (N-ary\
            image).
        transform_type: Type of transform to compute: 'rigid' or 'affine'.
        output_affine: The output file for the affine transform matrix (e.g.,\
            OutAffine.txt).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ANTSUseLandmarkImagesToGetAffineTransform",
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "transform_type": transform_type,
        "output_affine": output_affine,
    }
    return params


def antsuse_landmark_images_to_get_affine_transform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsuseLandmarkImagesToGetAffineTransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("fixed_image", None) is None:
        raise StyxValidationError("`fixed_image` must not be None")
    if not isinstance(params["fixed_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixed_image` has the wrong type: Received `{type(params.get("fixed_image", None))}` expected `InputPathType`')
    if params.get("moving_image", None) is None:
        raise StyxValidationError("`moving_image` must not be None")
    if not isinstance(params["moving_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image` has the wrong type: Received `{type(params.get("moving_image", None))}` expected `InputPathType`')
    if params.get("transform_type", None) is None:
        raise StyxValidationError("`transform_type` must not be None")
    if not isinstance(params["transform_type"], str):
        raise StyxValidationError(f'`transform_type` has the wrong type: Received `{type(params.get("transform_type", None))}` expected `typing.Literal["rigid", "affine"]`')
    if params["transform_type"] not in ["rigid", "affine"]:
        raise StyxValidationError("Parameter `transform_type` must be one of [\"rigid\", \"affine\"]")
    if params.get("output_affine", None) is None:
        raise StyxValidationError("`output_affine` must not be None")
    if not isinstance(params["output_affine"], str):
        raise StyxValidationError(f'`output_affine` has the wrong type: Received `{type(params.get("output_affine", None))}` expected `str`')


def antsuse_landmark_images_to_get_affine_transform_cargs(
    params: AntsuseLandmarkImagesToGetAffineTransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ANTSUseLandmarkImagesToGetAffineTransform")
    cargs.append(execution.input_file(params.get("fixed_image", None)))
    cargs.append(execution.input_file(params.get("moving_image", None)))
    cargs.append(params.get("transform_type", None))
    cargs.append(params.get("output_affine", None))
    return cargs


def antsuse_landmark_images_to_get_affine_transform_outputs(
    params: AntsuseLandmarkImagesToGetAffineTransformParamsDict,
    execution: Execution,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsuseLandmarkImagesToGetAffineTransformOutputs(
        root=execution.output_file("."),
        affine_transform_matrix=execution.output_file(params.get("output_affine", None)),
    )
    return ret


def antsuse_landmark_images_to_get_affine_transform_execute(
    params: AntsuseLandmarkImagesToGetAffineTransformParamsDict,
    runner: Runner | None = None,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    ANTSUseLandmarkImagesToGetAffineTransform
    
    This tool computes an affine transform (rigid or affine) from labeled
    landmark images. It requires input images to be N-ary, in the same physical
    space as the images you want to register, and to have the same landmark
    points defined within them. Landmarks are defined from the center of mass of
    the labels in the input images. ITK-snap can be used to generate the label
    images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetAffineTransformOutputs`).
    """
    antsuse_landmark_images_to_get_affine_transform_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA)
    params = execution.params(params)
    cargs = antsuse_landmark_images_to_get_affine_transform_cargs(params, execution)
    ret = antsuse_landmark_images_to_get_affine_transform_outputs(params, execution)
    execution.run(cargs)
    return ret


def antsuse_landmark_images_to_get_affine_transform(
    fixed_image: InputPathType,
    moving_image: InputPathType,
    transform_type: typing.Literal["rigid", "affine"],
    output_affine: str,
    runner: Runner | None = None,
) -> AntsuseLandmarkImagesToGetAffineTransformOutputs:
    """
    ANTSUseLandmarkImagesToGetAffineTransform
    
    This tool computes an affine transform (rigid or affine) from labeled
    landmark images. It requires input images to be N-ary, in the same physical
    space as the images you want to register, and to have the same landmark
    points defined within them. Landmarks are defined from the center of mass of
    the labels in the input images. ITK-snap can be used to generate the label
    images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        fixed_image: The fixed image containing labeled landmarks (N-ary\
            image).
        moving_image: The moving image containing labeled landmarks (N-ary\
            image).
        transform_type: Type of transform to compute: 'rigid' or 'affine'.
        output_affine: The output file for the affine transform matrix (e.g.,\
            OutAffine.txt).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsuseLandmarkImagesToGetAffineTransformOutputs`).
    """
    params = antsuse_landmark_images_to_get_affine_transform_params(
        fixed_image=fixed_image,
        moving_image=moving_image,
        transform_type=transform_type,
        output_affine=output_affine,
    )
    return antsuse_landmark_images_to_get_affine_transform_execute(params, runner)


__all__ = [
    "ANTSUSE_LANDMARK_IMAGES_TO_GET_AFFINE_TRANSFORM_METADATA",
    "AntsuseLandmarkImagesToGetAffineTransformOutputs",
    "AntsuseLandmarkImagesToGetAffineTransformParamsDict",
    "AntsuseLandmarkImagesToGetAffineTransformParamsDictTagged",
    "antsuse_landmark_images_to_get_affine_transform",
    "antsuse_landmark_images_to_get_affine_transform_execute",
    "antsuse_landmark_images_to_get_affine_transform_params",
]
